@@ -99,7 +99,7 @@ typedef struct {
 
 typedef struct {
 	int x,y,w,h;
-	unsigned long wid;
+	void *wid;
 	bool_t show;
 	bool_t refresh;
 } VideoParams;
