@@ -93,6 +93,14 @@ static void add_rtcp_fb_trr_int_attribute(belle_sdp_media_description_t *media_d
 	belle_sdp_media_description_add_attribute(media_desc, BELLE_SDP_ATTRIBUTE(attribute));
 }
 
+static void add_rtcp_fb_ack_attribute(belle_sdp_media_description_t *media_desc, int8_t id, belle_sdp_rtcp_fb_val_param_t param) {
+	belle_sdp_rtcp_fb_attribute_t *attribute = belle_sdp_rtcp_fb_attribute_new();
+	belle_sdp_rtcp_fb_attribute_set_id(attribute, id);
+	belle_sdp_rtcp_fb_attribute_set_type(attribute, BELLE_SDP_RTCP_FB_ACK);
+	belle_sdp_rtcp_fb_attribute_set_param(attribute, param);
+	belle_sdp_media_description_add_attribute(media_desc, BELLE_SDP_ATTRIBUTE(attribute));
+}
+
 static void add_rtcp_fb_nack_attribute(belle_sdp_media_description_t *media_desc, int8_t id, belle_sdp_rtcp_fb_val_param_t param) {
 	belle_sdp_rtcp_fb_attribute_t *attribute = belle_sdp_rtcp_fb_attribute_new();
 	belle_sdp_rtcp_fb_attribute_set_id(attribute, id);
@@ -120,6 +128,12 @@ static void add_rtcp_fb_attributes(belle_sdp_media_description_t *media_desc, co
 	if (general_trr_int == TRUE) {
 		add_rtcp_fb_trr_int_attribute(media_desc, -1, trr_int);
 	}
+	if (stream->rtcp_fb.generic_nack_enabled == TRUE) {
+		add_rtcp_fb_nack_attribute(media_desc, -1, BELLE_SDP_RTCP_FB_NONE);
+	}
+	if (stream->rtcp_fb.tmmbr_enabled == TRUE) {
+		add_rtcp_fb_ccm_attribute(media_desc, -1, BELLE_SDP_RTCP_FB_TMMBR);
+	}
 
 	for (pt_it = stream->payloads; pt_it != NULL; pt_it = pt_it->next) {
 		pt = (PayloadType *)pt_it->data;
@@ -128,6 +142,11 @@ static void add_rtcp_fb_attributes(belle_sdp_media_description_t *media_desc, co
 		payload_type_set_flag(pt, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
 		avpf_params = payload_type_get_avpf_params(pt);
 
+		/* Add trr-int if not set generally. */
+		if (general_trr_int != TRUE) {
+			add_rtcp_fb_trr_int_attribute(media_desc, payload_type_get_number(pt), avpf_params.trr_interval);
+		}
+
 		/* Add rtcp-fb attributes according to the AVPF features of the payload types. */
 		if (avpf_params.features & PAYLOAD_TYPE_AVPF_PLI) {
 			add_rtcp_fb_nack_attribute(media_desc, payload_type_get_number(pt), BELLE_SDP_RTCP_FB_PLI);
@@ -136,7 +155,11 @@ static void add_rtcp_fb_attributes(belle_sdp_media_description_t *media_desc, co
 			add_rtcp_fb_nack_attribute(media_desc, payload_type_get_number(pt), BELLE_SDP_RTCP_FB_SLI);
 		}
 		if (avpf_params.features & PAYLOAD_TYPE_AVPF_RPSI) {
-			add_rtcp_fb_nack_attribute(media_desc, payload_type_get_number(pt), BELLE_SDP_RTCP_FB_RPSI);
+			if (avpf_params.rpsi_compatibility == TRUE) {
+				add_rtcp_fb_nack_attribute(media_desc, payload_type_get_number(pt), BELLE_SDP_RTCP_FB_RPSI);
+			} else {
+				add_rtcp_fb_ack_attribute(media_desc, payload_type_get_number(pt), BELLE_SDP_RTCP_FB_RPSI);
+			}
 		}
 		if (avpf_params.features & PAYLOAD_TYPE_AVPF_FIR) {
 			add_rtcp_fb_ccm_attribute(media_desc, payload_type_get_number(pt), BELLE_SDP_RTCP_FB_FIR);
@@ -227,7 +250,7 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 	if ( stream->bandwidth>0 )
 		belle_sdp_media_description_set_bandwidth ( media_desc,"AS",stream->bandwidth );
 
-	if ((stream->proto == SalProtoRtpSavpf) || (stream->proto == SalProtoRtpSavp)) {
+	if (sal_stream_description_has_srtp(stream)) {
 		/* add crypto lines */
 		for ( j=0; j<SAL_CRYPTO_ALGO_MAX; j++ ) {
 			MSCryptoSuiteNameParams desc;
@@ -244,7 +267,7 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 
 	/* insert DTLS session attribute if needed */
 	if ((stream->proto == SalProtoUdpTlsRtpSavpf) || (stream->proto == SalProtoUdpTlsRtpSavp)) {
-		char* ssrc_attribute = ms_strdup_printf("%u cname:%s",htonl(stream->rtp_ssrc),stream->rtcp_cname);
+		char* ssrc_attribute = ms_strdup_printf("%u cname:%s",stream->rtp_ssrc,stream->rtcp_cname);
 		if ((stream->dtls_role != SalDtlsRoleInvalid) && (strlen(stream->dtls_fingerprint)>0)) {
 			switch(stream->dtls_role) {
 				case SalDtlsRoleIsClient:
@@ -260,7 +283,7 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 			}
 			belle_sdp_media_description_add_attribute(media_desc, belle_sdp_attribute_create("fingerprint",stream->dtls_fingerprint));
 		}
-		belle_sdp_media_description_add_attribute(media_desc, belle_sdp_attribute_create("ssrc",ssrc_attribute)); /* truc de Jehan a virer? */
+		belle_sdp_media_description_add_attribute(media_desc, belle_sdp_attribute_create("ssrc",ssrc_attribute));
 		ms_free(ssrc_attribute);
 	}
 
@@ -308,7 +331,7 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 		}
 	}
 
-	if ((rtp_port != 0) && ((stream->proto == SalProtoRtpAvpf) || (stream->proto == SalProtoRtpSavpf))) {
+	if ((rtp_port != 0) && sal_stream_description_has_avpf(stream)) {
 		add_rtcp_fb_attributes(media_desc, md, stream);
 	}
 
@@ -332,6 +355,16 @@ static void stream_description_to_sdp ( belle_sdp_session_description_t *session
 			belle_sip_object_unref((belle_sip_object_t*)media_attribute);
 		}
 	}
+
+	if (stream->custom_sdp_attributes) {
+		belle_sdp_session_description_t *custom_desc = (belle_sdp_session_description_t *)stream->custom_sdp_attributes;
+		belle_sip_list_t *l = belle_sdp_session_description_get_attributes(custom_desc);
+		belle_sip_list_t *elem;
+		for (elem = l; elem != NULL; elem = elem->next) {
+			belle_sdp_media_description_add_attribute(media_desc, (belle_sdp_attribute_t *)elem->data);
+		}
+	}
+
 	/*
 	 * rfc5576
 	 * 4.1.  The "ssrc" Media Attribute
@@ -392,6 +425,14 @@ belle_sdp_session_description_t * media_description_to_sdp ( const SalMediaDescr
 	if (desc->rtcp_xr.enabled == TRUE) {
 		belle_sdp_session_description_add_attribute(session_desc, create_rtcp_xr_attribute(&desc->rtcp_xr));
 	}
+	if (desc->custom_sdp_attributes) {
+		belle_sdp_session_description_t *custom_desc = (belle_sdp_session_description_t *)desc->custom_sdp_attributes;
+		belle_sip_list_t *l = belle_sdp_session_description_get_attributes(custom_desc);
+		belle_sip_list_t *elem;
+		for (elem = l; elem != NULL; elem = elem->next) {
+			belle_sdp_session_description_add_attribute(session_desc, (belle_sdp_attribute_t *)elem->data);
+		}
+	}
 
 	for ( i=0; i<desc->nb_streams; i++ ) {
 		stream_description_to_sdp(session_desc, desc, &desc->streams[i]);
@@ -533,14 +574,16 @@ static void enable_avpf_for_stream(SalStreamDescription *stream) {
 	}
 }
 
-static void apply_rtcp_fb_attribute_to_payload(belle_sdp_rtcp_fb_attribute_t *fb_attribute, PayloadType *pt) {
+static void apply_rtcp_fb_attribute_to_payload(belle_sdp_rtcp_fb_attribute_t *fb_attribute, SalStreamDescription *stream, PayloadType *pt) {
 	PayloadTypeAvpfParams avpf_params = payload_type_get_avpf_params(pt);
 	switch (belle_sdp_rtcp_fb_attribute_get_type(fb_attribute)) {
+		case BELLE_SDP_RTCP_FB_ACK:
+			if (belle_sdp_rtcp_fb_attribute_get_param(fb_attribute) == BELLE_SDP_RTCP_FB_RPSI) {
+				avpf_params.features |= PAYLOAD_TYPE_AVPF_RPSI;
+			}
+			break;
 		case BELLE_SDP_RTCP_FB_NACK:
 			switch (belle_sdp_rtcp_fb_attribute_get_param(fb_attribute)) {
-				case BELLE_SDP_RTCP_FB_NONE:
-					avpf_params.features |= PAYLOAD_TYPE_AVPF_PLI | PAYLOAD_TYPE_AVPF_SLI | PAYLOAD_TYPE_AVPF_RPSI;
-					break;
 				case BELLE_SDP_RTCP_FB_PLI:
 					avpf_params.features |= PAYLOAD_TYPE_AVPF_PLI;
 					break;
@@ -548,7 +591,14 @@ static void apply_rtcp_fb_attribute_to_payload(belle_sdp_rtcp_fb_attribute_t *fb
 					avpf_params.features |= PAYLOAD_TYPE_AVPF_SLI;
 					break;
 				case BELLE_SDP_RTCP_FB_RPSI:
+					/* Linphone uses positive feeback for RPSI. However first versions handling
+					 * AVPF wrongly declared RPSI as negative feedback, so this is kept for compatibility
+					 * with these versions but will probably be removed at some point in time. */
 					avpf_params.features |= PAYLOAD_TYPE_AVPF_RPSI;
+					avpf_params.rpsi_compatibility = TRUE;
+					break;
+				case BELLE_SDP_RTCP_FB_NONE:
+					stream->rtcp_fb.generic_nack_enabled = TRUE;
 					break;
 				default:
 					break;
@@ -562,11 +612,13 @@ static void apply_rtcp_fb_attribute_to_payload(belle_sdp_rtcp_fb_attribute_t *fb
 				case BELLE_SDP_RTCP_FB_FIR:
 					avpf_params.features |= PAYLOAD_TYPE_AVPF_FIR;
 					break;
+				case BELLE_SDP_RTCP_FB_TMMBR:
+					stream->rtcp_fb.tmmbr_enabled = TRUE;
+					break;
 				default:
 					break;
 			}
 			break;
-		case BELLE_SDP_RTCP_FB_ACK:
 		default:
 			break;
 	}
@@ -589,7 +641,7 @@ static void sdp_parse_rtcp_fb_parameters(belle_sdp_media_description_t *media_de
 			if (belle_sdp_rtcp_fb_attribute_get_id(fb_attribute) == -1) {
 				for (pt_it = stream->payloads; pt_it != NULL; pt_it = pt_it->next) {
 					pt = (PayloadType *)pt_it->data;
-					apply_rtcp_fb_attribute_to_payload(fb_attribute, pt);
+					apply_rtcp_fb_attribute_to_payload(fb_attribute, stream, pt);
 				}
 			}
 		}
@@ -604,7 +656,7 @@ static void sdp_parse_rtcp_fb_parameters(belle_sdp_media_description_t *media_de
 			for (pt_it = stream->payloads; pt_it != NULL; pt_it = pt_it->next) {
 				pt = (PayloadType *)pt_it->data;
 				if (payload_type_get_number(pt) == (int)pt_num) {
-					apply_rtcp_fb_attribute_to_payload(fb_attribute, pt);
+					apply_rtcp_fb_attribute_to_payload(fb_attribute, stream, pt);
 				}
 			}
 		}
@@ -668,6 +720,7 @@ static SalStreamDescription * sdp_to_stream_description(SalMediaDescription *md,
 	belle_sdp_connection_t* cnx;
 	belle_sdp_media_t* media;
 	belle_sdp_attribute_t* attribute;
+	belle_sip_list_t *custom_attribute_it;
 	const char* value;
 	const char *mtype,*proto;
 
@@ -705,6 +758,8 @@ static SalStreamDescription * sdp_to_stream_description(SalMediaDescription *md,
 		stream->type=SalAudio;
 	} else if ( strcasecmp ( "video", mtype ) == 0 ) {
 		stream->type=SalVideo;
+	} else if ( strcasecmp ( "text", mtype ) == 0 ) {
+		stream->type=SalText;
 	} else {
 		stream->type=SalOther;
 		strncpy ( stream->typeother,mtype,sizeof ( stream->typeother )-1 );
@@ -763,7 +818,7 @@ static SalStreamDescription * sdp_to_stream_description(SalMediaDescription *md,
 	}
 
 	/* Read crypto lines if any */
-	if ((stream->proto == SalProtoRtpSavpf) || (stream->proto == SalProtoRtpSavp)) {
+	if (sal_stream_description_has_srtp(stream)) {
 		sdp_parse_media_crypto_parameters(media_desc, stream);
 	}
 
@@ -771,7 +826,7 @@ static SalStreamDescription * sdp_to_stream_description(SalMediaDescription *md,
 	sdp_parse_media_ice_parameters(media_desc, stream);
 
 	/* Get RTCP-FB attributes if any */
-	if ((stream->proto == SalProtoRtpAvpf) || (stream->proto == SalProtoRtpSavpf)) {
+	if (sal_stream_description_has_avpf(stream)) {
 		enable_avpf_for_stream(stream);
 		sdp_parse_rtcp_fb_parameters(media_desc, stream);
 	}
@@ -780,6 +835,12 @@ static SalStreamDescription * sdp_to_stream_description(SalMediaDescription *md,
 	stream->rtcp_xr = md->rtcp_xr;	// Use session parameters if no stream parameters are defined
 	sdp_parse_media_rtcp_xr_parameters(media_desc, &stream->rtcp_xr);
 
+	/* Get the custom attributes */
+	for (custom_attribute_it = belle_sdp_media_description_get_attributes(media_desc); custom_attribute_it != NULL; custom_attribute_it = custom_attribute_it->next) {
+		belle_sdp_attribute_t *attr = (belle_sdp_attribute_t *)custom_attribute_it->data;
+		stream->custom_sdp_attributes = sal_custom_sdp_attribute_append(stream->custom_sdp_attributes, belle_sdp_attribute_get_name(attr), belle_sdp_attribute_get_value(attr));
+	}
+
 	md->nb_streams++;
 	return stream;
 }
@@ -790,6 +851,7 @@ int sdp_to_media_description ( belle_sdp_session_description_t  *session_desc, S
 	belle_sip_list_t* media_desc_it;
 	belle_sdp_media_description_t* media_desc;
 	belle_sdp_session_name_t *sname;
+	belle_sip_list_t *custom_attribute_it;
 	const char* value;
 	SalDtlsRole session_role=SalDtlsRoleInvalid;
 	int i;
@@ -851,6 +913,12 @@ int sdp_to_media_description ( belle_sdp_session_description_t  *session_desc, S
 	/* Get session RTCP-XR attributes if any */
 	sdp_parse_session_rtcp_xr_parameters(session_desc, &desc->rtcp_xr);
 
+	/* Get the custom attributes */
+	for (custom_attribute_it = belle_sdp_session_description_get_attributes(session_desc); custom_attribute_it != NULL; custom_attribute_it = custom_attribute_it->next) {
+		belle_sdp_attribute_t *attr = (belle_sdp_attribute_t *)custom_attribute_it->data;
+		desc->custom_sdp_attributes = sal_custom_sdp_attribute_append(desc->custom_sdp_attributes, belle_sdp_attribute_get_name(attr), belle_sdp_attribute_get_value(attr));
+	}
+
 	for ( media_desc_it=belle_sdp_session_description_get_media_descriptions ( session_desc )
 						; media_desc_it!=NULL
 			; media_desc_it=media_desc_it->next ) {
