@@ -25,7 +25,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define PRIu64 "I64u"
 #endif
 
-#ifndef WIN32
+#ifndef _WIN32
 #if !defined(ANDROID) && !defined(__QNXNTO__)
 #	include <langinfo.h>
 #	include <iconv.h>
@@ -38,6 +38,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define MAX_PATH_SIZE 1024
 
 #include "sqlite3.h"
+#include <assert.h>
 
 static ORTP_INLINE LinphoneChatMessage* get_transient_message(LinphoneChatRoom* cr, unsigned int storage_id){
 	MSList* transients = cr->transient_messages;
@@ -93,6 +94,20 @@ static void fetch_content_from_database(sqlite3 *db, LinphoneChatMessage *messag
 	sqlite3_free(buf);
 }
 
+
+
+// Called when fetching all conversations from database
+static int callback_all(void *data, int argc, char **argv, char **colName){
+	LinphoneCore* lc = (LinphoneCore*) data;
+	char* address = argv[0];
+	LinphoneAddress *addr = linphone_address_new(address);
+	if (addr){
+		linphone_core_get_chat_room(lc, addr);
+		linphone_address_destroy(addr);
+	}
+	return 0;
+}
+
 /* DB layout:
  * | 0  | storage_id
  * | 1  | localContact
@@ -105,13 +120,12 @@ static void fetch_content_from_database(sqlite3 *db, LinphoneChatMessage *messag
  * | 8  | external body url
  * | 9  | utc timestamp
  * | 10 | app data text
- * | 11 | linphone content
+ * | 11 | linphone content id
  */
-static void create_chat_message(char **argv, void *data){
+static int create_chat_message(void *data, int argc, char **argv, char **colName){
 	LinphoneChatRoom *cr = (LinphoneChatRoom *)data;
 	LinphoneAddress *from;
 	LinphoneAddress *to;
-
 	unsigned int storage_id = atoi(argv[0]);
 
 	// check if the message exists in the transient list, in which case we should return that one.
@@ -135,17 +149,12 @@ static void create_chat_message(char **argv, void *data){
 			linphone_address_destroy(to);
 		}
 
-		if( argv[9] != NULL ){
-			new_message->time = (time_t)atol(argv[9]);
-		} else {
-			new_message->time = time(NULL);
-		}
-
+		new_message->time = (time_t)atol(argv[9]);
 		new_message->is_read=atoi(argv[6]);
 		new_message->state=atoi(argv[7]);
 		new_message->storage_id=storage_id;
-		new_message->external_body_url= argv[8] ? ms_strdup(argv[8])  : NULL;
-		new_message->appdata          = argv[10]? ms_strdup(argv[10]) : NULL;
+		new_message->external_body_url= ms_strdup(argv[8]);
+		new_message->appdata          = ms_strdup(argv[10]);
 
 		if (argv[11] != NULL) {
 			int id = atoi(argv[11]);
@@ -155,25 +164,14 @@ static void create_chat_message(char **argv, void *data){
 		}
 	}
 	cr->messages_hist=ms_list_prepend(cr->messages_hist,new_message);
-}
 
-// Called when fetching all conversations from database
-static int callback_all(void *data, int argc, char **argv, char **colName){
-	LinphoneCore* lc = (LinphoneCore*) data;
-	char* address = argv[0];
-	linphone_core_get_or_create_chat_room(lc, address);
-	return 0;
-}
-
-static int callback(void *data, int argc, char **argv, char **colName){
-	create_chat_message(argv,data);
 	return 0;
 }
 
 void linphone_sql_request_message(sqlite3 *db,const char *stmt,LinphoneChatRoom *cr){
 	char* errmsg=NULL;
 	int ret;
-	ret=sqlite3_exec(db,stmt,callback,cr,&errmsg);
+	ret=sqlite3_exec(db,stmt,create_chat_message,cr,&errmsg);
 	if(ret != SQLITE_OK) {
 		ms_error("Error in creation: %s.", errmsg);
 		sqlite3_free(errmsg);
@@ -287,12 +285,17 @@ void linphone_chat_room_mark_as_read(LinphoneChatRoom *cr){
 
 	if (lc->db==NULL) return ;
 
+	// optimization: do not modify the database if no message is marked as unread
+	if(linphone_chat_room_get_unread_messages_count(cr) == 0) return;
+
 	peer=linphone_address_as_string_uri_only(linphone_chat_room_get_peer_address(cr));
 	buf=sqlite3_mprintf("UPDATE history SET read=%i WHERE remoteContact = %Q;",
 				   read,peer);
 	linphone_sql_request(lc->db,buf);
 	sqlite3_free(buf);
 	ms_free(peer);
+
+	cr->unread_count = 0;
 }
 
 void linphone_chat_room_update_url(LinphoneChatRoom *cr, LinphoneChatMessage *msg) {
@@ -316,6 +319,9 @@ static int linphone_chat_room_get_messages_count(LinphoneChatRoom *cr, bool_t un
 
 	if (lc->db==NULL) return 0;
 
+	// optimization: do not read database if the count is already available in memory
+	if(unread_only && cr->unread_count >= 0) return cr->unread_count;
+
 	peer=linphone_address_as_string_uri_only(linphone_chat_room_get_peer_address(cr));
 	buf=sqlite3_mprintf("SELECT count(*) FROM history WHERE remoteContact = %Q %s;",peer,unread_only?"AND read = 0":"");
 	returnValue = sqlite3_prepare_v2(lc->db,buf,-1,&selectStatement,NULL);
@@ -327,6 +333,11 @@ static int linphone_chat_room_get_messages_count(LinphoneChatRoom *cr, bool_t un
 	sqlite3_finalize(selectStatement);
 	sqlite3_free(buf);
 	ms_free(peer);
+
+	/* no need to test the sign of cr->unread_count here
+	 * because it has been tested above */
+	if(unread_only) cr->unread_count = numrows;
+
 	return numrows;
 }
 
@@ -347,6 +358,11 @@ void linphone_chat_room_delete_message(LinphoneChatRoom *cr, LinphoneChatMessage
 	buf=sqlite3_mprintf("DELETE FROM history WHERE id = %i;", msg->storage_id);
 	linphone_sql_request(lc->db,buf);
 	sqlite3_free(buf);
+
+	if(cr->unread_count >= 0 && !msg->is_read) {
+		assert(cr->unread_count > 0);
+		cr->unread_count--;
+	}
 }
 
 void linphone_chat_room_delete_history(LinphoneChatRoom *cr){
@@ -361,6 +377,8 @@ void linphone_chat_room_delete_history(LinphoneChatRoom *cr){
 	linphone_sql_request(lc->db,buf);
 	sqlite3_free(buf);
 	ms_free(peer);
+
+	if(cr->unread_count > 0) cr->unread_count = 0;
 }
 
 MSList *linphone_chat_room_get_history_range(LinphoneChatRoom *cr, int startm, int endm){
@@ -415,11 +433,6 @@ MSList *linphone_chat_room_get_history(LinphoneChatRoom *cr,int nb_message){
 	return linphone_chat_room_get_history_range(cr, 0, nb_message-1);
 }
 
-
-void linphone_close_storage(sqlite3* db){
-	sqlite3_close(db);
-}
-
 void linphone_create_table(sqlite3* db){
 	char* errmsg=NULL;
 	int ret;
@@ -592,21 +605,22 @@ void linphone_core_message_storage_set_debug(LinphoneCore *lc, bool_t debug){
 }
 
 static int _linphone_sqlite3_open(const char *db_file, sqlite3 **db) {
-#if defined(ANDROID) || defined(__QNXNTO__)
-	return sqlite3_open(db_file, db);
-#elif defined(WIN32)
+	char* errmsg = NULL;
 	int ret;
+#if defined(ANDROID) || defined(__QNXNTO__)
+	ret = sqlite3_open(db_file, db);
+#elif defined(_WIN32)
 	wchar_t db_file_utf16[MAX_PATH_SIZE];
-	ret = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, db_file, MAX_PATH_SIZE, db_file_utf16, MAX_PATH_SIZE);
+	ret = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, db_file, -1, db_file_utf16, MAX_PATH_SIZE);
 	if(ret == 0) db_file_utf16[0] = '\0';
-	return sqlite3_open16(db_file_utf16, db);
+	ret = sqlite3_open16(db_file_utf16, db);
 #else
 	char db_file_locale[MAX_PATH_SIZE] = {'\0'};
 	char db_file_utf8[MAX_PATH_SIZE] = "";
 	char *inbuf=db_file_locale, *outbuf=db_file_utf8;
 	size_t inbyteleft = MAX_PATH_SIZE, outbyteleft = MAX_PATH_SIZE;
 	iconv_t cb;
-	
+
 	strncpy(db_file_locale, db_file, MAX_PATH_SIZE-1);
 	cb = iconv_open("UTF-8", nl_langinfo(CODESET));
 	if(cb != (iconv_t)-1) {
@@ -615,8 +629,18 @@ static int _linphone_sqlite3_open(const char *db_file, sqlite3 **db) {
 		if(ret == -1) db_file_utf8[0] = '\0';
 		iconv_close(cb);
 	}
-	return sqlite3_open(db_file_utf8, db);
+	ret = sqlite3_open(db_file_utf8, db);
 #endif
+	if (ret != SQLITE_OK) return ret;
+	// Some platforms do not provide a way to create temporary files which are needed
+	// for transactions... so we work in memory only
+	// see http ://www.sqlite.org/compile.html#temp_store
+	ret = sqlite3_exec(*db, "PRAGMA temp_store=MEMORY", NULL, NULL, &errmsg);
+	if (ret != SQLITE_OK) {
+		ms_error("Cannot set sqlite3 temporary store to memory: %s.", errmsg);
+		sqlite3_free(errmsg);
+	}
+	return ret;
 }
 
 void linphone_core_message_storage_init(LinphoneCore *lc){
