@@ -170,7 +170,9 @@ static MSList *match_payloads(const MSList *local, const MSList *remote, bool_t
 			newp->flags|=PAYLOAD_TYPE_FLAG_CAN_RECV|PAYLOAD_TYPE_FLAG_CAN_SEND;
 			if (p2->flags & PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED) {
 				newp->flags |= PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED;
-				newp->avpf = payload_type_get_avpf_params(p2); /* Take remote AVPF features */
+				/* Negotiation of AVPF features (keep common features) */
+				newp->avpf.features &= p2->avpf.features;
+				newp->avpf.rpsi_compatibility = p2->avpf.rpsi_compatibility;
 				/* Take bigger AVPF trr interval */
 				if (p2->avpf.trr_interval < matched->avpf.trr_interval) {
 					newp->avpf.trr_interval = matched->avpf.trr_interval;
@@ -419,7 +421,7 @@ static void initiate_incoming(const SalStreamDescription *local_cap,
 	result->proto=remote_offer->proto;
 	result->type=local_cap->type;
 	result->dir=compute_dir_incoming(local_cap->dir,remote_offer->dir);
-	if (!result->payloads || only_telephone_event(result->payloads) || remote_offer->rtp_port==0 || remote_offer->dir==SalStreamRecvOnly){
+	if (!result->payloads || only_telephone_event(result->payloads) || remote_offer->rtp_port==0){
 		result->rtp_port=0;
 		return;
 	}
@@ -488,26 +490,29 @@ static void initiate_incoming(const SalStreamDescription *local_cap,
 int offer_answer_initiate_outgoing(const SalMediaDescription *local_offer,
 					const SalMediaDescription *remote_answer,
 					SalMediaDescription *result){
-	int i,j;
+	int i;
 	const SalStreamDescription *ls,*rs;
 
-	for(i=0,j=0;i<local_offer->nb_streams;++i){
+	for(i=0;i<local_offer->nb_streams;++i){
 		ms_message("Processing for stream %i",i);
 		ls=&local_offer->streams[i];
-		rs=sal_media_description_find_stream((SalMediaDescription*)remote_answer,ls->proto,ls->type);
-		if (rs) {
-			initiate_outgoing(ls,rs,&result->streams[j]);
+		rs=&remote_answer->streams[i];
+		if (rs && ls->proto == rs->proto && rs->type == ls->type) {
+			initiate_outgoing(ls,rs,&result->streams[i]);
 			memcpy(&result->streams[i].rtcp_xr, &ls->rtcp_xr, sizeof(result->streams[i].rtcp_xr));
 			if ((ls->rtcp_xr.enabled == TRUE) && (rs->rtcp_xr.enabled == FALSE)) {
 				result->streams[i].rtcp_xr.enabled = FALSE;
 			}
-			++j;
+			result->streams[i].rtcp_fb.generic_nack_enabled = ls->rtcp_fb.generic_nack_enabled & rs->rtcp_fb.generic_nack_enabled;
+			result->streams[i].rtcp_fb.tmmbr_enabled = ls->rtcp_fb.tmmbr_enabled & rs->rtcp_fb.tmmbr_enabled;
 		}
 		else ms_warning("No matching stream for %i",i);
 	}
 	result->nb_streams=local_offer->nb_streams;
 	result->bandwidth=remote_answer->bandwidth;
 	strcpy(result->addr,remote_answer->addr);
+	strcpy(result->ice_pwd, local_offer->ice_pwd);
+	strcpy(result->ice_ufrag, local_offer->ice_ufrag);
 	memcpy(&result->rtcp_xr, &local_offer->rtcp_xr, sizeof(result->rtcp_xr));
 	if ((local_offer->rtcp_xr.enabled == TRUE) && (remote_answer->rtcp_xr.enabled == FALSE)) {
 		result->rtcp_xr.enabled = FALSE;
@@ -516,39 +521,6 @@ int offer_answer_initiate_outgoing(const SalMediaDescription *local_offer,
 	return 0;
 }
 
-static bool_t local_stream_not_already_used(const SalMediaDescription *result, const SalStreamDescription *stream){
-	int i;
-	for(i=0;i<result->nb_streams;++i){
-		const SalStreamDescription *ss=&result->streams[i];
-		if (strcmp(ss->name,stream->name)==0){
-			ms_message("video stream already used in answer");
-			return FALSE;
-		}
-	}
-	return TRUE;
-}
-
-/*in answering mode, we consider that if we are able to make AVPF/SAVP/SAVPF, then we can do AVP as well*/
-static bool_t proto_compatible(SalMediaProto local, SalMediaProto remote) {
-	if (local == remote) return TRUE;
-	if ((remote == SalProtoRtpAvp) && ((local == SalProtoRtpSavp) || (local == SalProtoRtpSavpf))) return TRUE;
-	if ((remote == SalProtoRtpAvp) && ((local == SalProtoUdpTlsRtpSavp) || (local == SalProtoUdpTlsRtpSavpf))) return TRUE;
-	if ((remote == SalProtoRtpAvpf) && (local == SalProtoRtpSavpf)) return TRUE;
-	if ((remote == SalProtoRtpAvpf) && (local == SalProtoUdpTlsRtpSavpf)) return TRUE;
-	return FALSE;
-}
-
-static const SalStreamDescription *find_local_matching_stream(const SalMediaDescription *result, const SalMediaDescription *local_capabilities, const SalStreamDescription *remote_stream){
-	int i;
-	for(i=0;i<local_capabilities->nb_streams;++i){
-		const SalStreamDescription *ss=&local_capabilities->streams[i];
-		if (!sal_stream_description_active(ss)) continue;
-		if (ss->type==remote_stream->type && proto_compatible(ss->proto,remote_stream->proto)
-			&& local_stream_not_already_used(result,ss)) return ss;
-	}
-	return NULL;
-}
-
 /**
  * Returns a media description to run the streams with, based on the local capabilities and
  * and the received offer.
@@ -561,12 +533,13 @@ int offer_answer_initiate_incoming(const SalMediaDescription *local_capabilities
 	const SalStreamDescription *ls=NULL,*rs;
 
 	for(i=0;i<remote_offer->nb_streams;++i){
-		rs=&remote_offer->streams[i];
-		if (rs->proto!=SalProtoOther){
-			ls=find_local_matching_stream(result,local_capabilities,rs);
-		}else ms_warning("Unknown protocol for mline %i, declining",i);
-		if (ls){
+		rs = &remote_offer->streams[i];
+		ls = &local_capabilities->streams[i];
+		if (ls && rs->type == ls->type && rs->proto == ls->proto){
 			initiate_incoming(ls,rs,&result->streams[i],one_matching_codec);
+			// Handle global RTCP FB attributes
+			result->streams[i].rtcp_fb.generic_nack_enabled = rs->rtcp_fb.generic_nack_enabled;
+			result->streams[i].rtcp_fb.tmmbr_enabled = rs->rtcp_fb.tmmbr_enabled;
 			// Handle media RTCP XR attribute
 			memset(&result->streams[i].rtcp_xr, 0, sizeof(result->streams[i].rtcp_xr));
 			if (rs->rtcp_xr.enabled == TRUE) {
@@ -593,6 +566,7 @@ int offer_answer_initiate_incoming(const SalMediaDescription *local_capabilities
 				strncpy(result->streams[i].proto_other,rs->proto_other,sizeof(rs->proto_other)-1);
 			}
 		}
+		result->streams[i].custom_sdp_attributes = sal_custom_sdp_attribute_clone(ls->custom_sdp_attributes);
 	}
 	result->nb_streams=i;
 	strcpy(result->username, local_capabilities->username);
@@ -604,6 +578,7 @@ int offer_answer_initiate_incoming(const SalMediaDescription *local_capabilities
 	strcpy(result->ice_ufrag, local_capabilities->ice_ufrag);
 	result->ice_lite = local_capabilities->ice_lite;
 	result->ice_completed = local_capabilities->ice_completed;
+	result->custom_sdp_attributes = sal_custom_sdp_attribute_clone(local_capabilities->custom_sdp_attributes);
 
 	strcpy(result->name,local_capabilities->name);
 
