@@ -55,7 +55,13 @@ SalTransport sal_transport_parse(const char* param) {
 
 SalMediaDescription *sal_media_description_new(){
 	SalMediaDescription *md=ms_new0(SalMediaDescription,1);
+	int i;
 	md->refcount=1;
+	for(i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; i++) {
+		md->streams[i].dir=SalStreamInactive;
+		md->streams[i].rtp_port = 0;
+		md->streams[i].rtcp_port = 0;
+	}
 	return md;
 }
 
@@ -66,7 +72,9 @@ static void sal_media_description_destroy(SalMediaDescription *md){
 		ms_list_free_with_data(md->streams[i].already_assigned_payloads,(void (*)(void *))payload_type_destroy);
 		md->streams[i].payloads=NULL;
 		md->streams[i].already_assigned_payloads=NULL;
+		sal_custom_sdp_attribute_free(md->streams[i].custom_sdp_attributes);
 	}
+	sal_custom_sdp_attribute_free(md->custom_sdp_attributes);
 	ms_free(md);
 }
 
@@ -82,10 +90,9 @@ void sal_media_description_unref(SalMediaDescription *md){
 	}
 }
 
-SalStreamDescription *sal_media_description_find_stream(SalMediaDescription *md,
-	SalMediaProto proto, SalStreamType type){
+SalStreamDescription *sal_media_description_find_stream(SalMediaDescription *md, SalMediaProto proto, SalStreamType type){
 	int i;
-	for(i=0;i<md->nb_streams;++i){
+	for(i=0;i<SAL_MEDIA_DESCRIPTION_MAX_STREAMS;++i){
 		SalStreamDescription *ss=&md->streams[i];
 		if (!sal_stream_description_active(ss)) continue;
 		if (ss->proto==proto && ss->type==type) return ss;
@@ -96,7 +103,7 @@ SalStreamDescription *sal_media_description_find_stream(SalMediaDescription *md,
 unsigned int sal_media_description_nb_active_streams_of_type(SalMediaDescription *md, SalStreamType type) {
 	unsigned int i;
 	unsigned int nb = 0;
-	for (i = 0; i < md->nb_streams; ++i) {
+	for (i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; ++i) {
 		if (!sal_stream_description_active(&md->streams[i])) continue;
 		if (md->streams[i].type == type) nb++;
 	}
@@ -105,7 +112,7 @@ unsigned int sal_media_description_nb_active_streams_of_type(SalMediaDescription
 
 SalStreamDescription * sal_media_description_get_active_stream_of_type(SalMediaDescription *md, SalStreamType type, unsigned int idx) {
 	unsigned int i;
-	for (i = 0; i < md->nb_streams; ++i) {
+	for (i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; ++i) {
 		if (!sal_stream_description_active(&md->streams[i])) continue;
 		if (md->streams[i].type == type) {
 			if (idx-- == 0) return &md->streams[i];
@@ -137,7 +144,7 @@ bool_t sal_media_description_empty(const SalMediaDescription *md){
 
 void sal_media_description_set_dir(SalMediaDescription *md, SalStreamDir stream_dir){
 	int i;
-	for(i=0;i<md->nb_streams;++i){
+	for(i=0;i<SAL_MEDIA_DESCRIPTION_MAX_STREAMS;++i){
 		SalStreamDescription *ss=&md->streams[i];
 		if (!sal_stream_description_active(ss)) continue;
 		ss->dir=stream_dir;
@@ -147,13 +154,12 @@ void sal_media_description_set_dir(SalMediaDescription *md, SalStreamDir stream_
 int sal_media_description_get_nb_active_streams(const SalMediaDescription *md) {
 	int i;
 	int nb = 0;
-	for (i = 0; i < md->nb_streams; i++) {
+	for (i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; i++) {
 		if (sal_stream_description_active(&md->streams[i])) nb++;
 	}
 	return nb;
 }
 
-
 static bool_t is_null_address(const char *addr){
 	return strcmp(addr,"0.0.0.0")==0 || strcmp(addr,"::0")==0;
 }
@@ -161,26 +167,27 @@ static bool_t is_null_address(const char *addr){
 /*check for the presence of at least one stream with requested direction */
 static bool_t has_dir(const SalMediaDescription *md, SalStreamDir stream_dir){
 	int i;
-
+	
 	/* we are looking for at least one stream with requested direction, inactive streams are ignored*/
-	for(i=0;i<md->nb_streams;++i){
+	for(i=0;i<SAL_MEDIA_DESCRIPTION_MAX_STREAMS;++i){
 		const SalStreamDescription *ss=&md->streams[i];
 		if (!sal_stream_description_active(ss)) continue;
-		if (ss->dir==stream_dir) return TRUE;
-		/*compatibility check for phones that only used the null address and no attributes */
-		if (ss->dir==SalStreamSendRecv && stream_dir==SalStreamSendOnly && (is_null_address(md->addr) || is_null_address(ss->rtp_addr)))
+		if (ss->dir==stream_dir) {
 			return TRUE;
+		}
+		/*compatibility check for phones that only used the null address and no attributes */
+		if (ss->dir==SalStreamSendRecv && stream_dir==SalStreamSendOnly && (is_null_address(md->addr) || is_null_address(ss->rtp_addr))){
+			return TRUE;
+		}
 	}
 	return FALSE;
 }
 
 bool_t sal_media_description_has_dir(const SalMediaDescription *md, SalStreamDir stream_dir){
 	if (stream_dir==SalStreamRecvOnly){
-		if (has_dir(md,SalStreamSendOnly) || has_dir(md,SalStreamSendRecv)) return FALSE;
-		else return TRUE;
+		return has_dir(md, SalStreamRecvOnly) && !(has_dir(md,SalStreamSendOnly) || has_dir(md,SalStreamSendRecv));
 	}else if (stream_dir==SalStreamSendOnly){
-		if (has_dir(md,SalStreamRecvOnly) || has_dir(md,SalStreamSendRecv)) return FALSE;
-		else return TRUE;
+		return has_dir(md, SalStreamSendOnly) && !(has_dir(md,SalStreamRecvOnly) || has_dir(md,SalStreamSendRecv));
 	}else if (stream_dir==SalStreamSendRecv){
 		return has_dir(md,SalStreamSendRecv);
 	}else{
@@ -196,22 +203,57 @@ bool_t sal_stream_description_active(const SalStreamDescription *sd) {
 	return (sd->rtp_port > 0);
 }
 
+/*these are switch case, so that when a new proto is added we can't forget to modify this function*/
 bool_t sal_stream_description_has_avpf(const SalStreamDescription *sd) {
-	return ((sd->proto == SalProtoRtpAvpf) || (sd->proto == SalProtoRtpSavpf) || (sd->proto == SalProtoUdpTlsRtpSavpf));
+	switch (sd->proto){
+		case SalProtoRtpAvpf:
+		case SalProtoRtpSavpf:
+		case SalProtoUdpTlsRtpSavpf:
+			return TRUE;
+		case SalProtoRtpAvp:
+		case SalProtoRtpSavp:
+		case SalProtoUdpTlsRtpSavp:
+		case SalProtoOther:
+			return FALSE;
+	}
+	return FALSE;
 }
 
+/*these are switch case, so that when a new proto is added we can't forget to modify this function*/
 bool_t sal_stream_description_has_srtp(const SalStreamDescription *sd) {
-	return ((sd->proto == SalProtoRtpSavp) || (sd->proto == SalProtoRtpSavpf));
+	switch (sd->proto){
+		case SalProtoRtpSavp:
+		case SalProtoRtpSavpf:
+			return TRUE;
+		case SalProtoRtpAvp:
+		case SalProtoRtpAvpf:
+		case SalProtoUdpTlsRtpSavpf:
+		case SalProtoUdpTlsRtpSavp:
+		case SalProtoOther:
+			return FALSE;
+	}
+	return FALSE;
 }
 
 bool_t sal_stream_description_has_dtls(const SalStreamDescription *sd) {
-	return ((sd->proto == SalProtoUdpTlsRtpSavp) || (sd->proto == SalProtoUdpTlsRtpSavpf));
+	switch (sd->proto){
+		case SalProtoUdpTlsRtpSavpf:
+		case SalProtoUdpTlsRtpSavp:
+			return TRUE;
+		case SalProtoRtpSavp:
+		case SalProtoRtpSavpf:
+		case SalProtoRtpAvp:
+		case SalProtoRtpAvpf:
+		case SalProtoOther:
+			return FALSE;
+	}
+	return FALSE;
 }
 
 bool_t sal_media_description_has_avpf(const SalMediaDescription *md) {
 	int i;
 	if (md->nb_streams == 0) return FALSE;
-	for (i = 0; i < md->nb_streams; i++) {
+	for (i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; i++) {
 		if (!sal_stream_description_active(&md->streams[i])) continue;
 		if (sal_stream_description_has_avpf(&md->streams[i]) != TRUE) return FALSE;
 	}
@@ -221,7 +263,7 @@ bool_t sal_media_description_has_avpf(const SalMediaDescription *md) {
 bool_t sal_media_description_has_srtp(const SalMediaDescription *md) {
 	int i;
 	if (md->nb_streams == 0) return FALSE;
-	for (i = 0; i < md->nb_streams; i++) {
+	for (i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; i++) {
 		if (!sal_stream_description_active(&md->streams[i])) continue;
 		if (sal_stream_description_has_srtp(&md->streams[i]) != TRUE) return FALSE;
 	}
@@ -231,7 +273,7 @@ bool_t sal_media_description_has_srtp(const SalMediaDescription *md) {
 bool_t sal_media_description_has_dtls(const SalMediaDescription *md) {
 	int i;
 	if (md->nb_streams == 0) return FALSE;
-	for (i = 0; i < md->nb_streams; i++) {
+	for (i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; i++) {
 		if (!sal_stream_description_active(&md->streams[i])) continue;
 		if (sal_stream_description_has_dtls(&md->streams[i]) != TRUE) return FALSE;
 	}
@@ -307,6 +349,8 @@ int sal_stream_description_equals(const SalStreamDescription *sd1, const SalStre
 
 	if (sd1->type != sd2->type) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
 	if (strcmp(sd1->rtp_addr, sd2->rtp_addr) != 0) result |= SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED;
+	if (sd1->rtp_addr[0]!='\0' && sd2->rtp_addr[0]!='\0' && ms_is_multicast(sd1->rtp_addr) != ms_is_multicast(sd2->rtp_addr))
+			result |= SAL_MEDIA_DESCRIPTION_NETWORK_XXXCAST_CHANGED;
 	if (sd1->rtp_port != sd2->rtp_port) {
 		if ((sd1->rtp_port == 0) || (sd2->rtp_port == 0)) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
 		else result |= SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED;
@@ -318,6 +362,10 @@ int sal_stream_description_equals(const SalStreamDescription *sd1, const SalStre
 	if (sd1->ptime != sd2->ptime) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
 	if (sd1->dir != sd2->dir) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
 
+	/* ICE */
+	if (strcmp(sd1->ice_ufrag, sd2->ice_ufrag) != 0) result |= SAL_MEDIA_DESCRIPTION_ICE_RESTART_DETECTED;
+	if (strcmp(sd1->ice_pwd, sd2->ice_pwd) != 0) result |= SAL_MEDIA_DESCRIPTION_ICE_RESTART_DETECTED;
+
 	/*DTLS*/
 	if (sd1->dtls_role != sd2->dtls_role) result |= SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED;
 	if (strcmp(sd1->dtls_fingerprint, sd2->dtls_fingerprint) != 0) result |= SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED;
@@ -325,18 +373,68 @@ int sal_stream_description_equals(const SalStreamDescription *sd1, const SalStre
 	return result;
 }
 
+char * sal_media_description_print_differences(int result){
+	char *out = NULL;
+	if (result & SAL_MEDIA_DESCRIPTION_CODEC_CHANGED){
+		out = ms_strcat_printf(out, "%s ", "CODEC_CHANGED");
+		result &= ~SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
+	}
+	if (result & SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED){
+		out = ms_strcat_printf(out, "%s ", "NETWORK_CHANGED");
+		result &= ~SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED;
+	}
+	if (result & SAL_MEDIA_DESCRIPTION_ICE_RESTART_DETECTED){
+		out = ms_strcat_printf(out, "%s ", "ICE_RESTART_DETECTED");
+		result &= ~SAL_MEDIA_DESCRIPTION_ICE_RESTART_DETECTED;
+	}
+	if (result & SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED){
+		out = ms_strcat_printf(out, "%s ", "CRYPTO_KEYS_CHANGED");
+		result &= ~SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED;
+	}
+	if (result & SAL_MEDIA_DESCRIPTION_NETWORK_XXXCAST_CHANGED){
+		out = ms_strcat_printf(out, "%s ", "NETWORK_XXXCAST_CHANGED");
+		result &= ~SAL_MEDIA_DESCRIPTION_NETWORK_XXXCAST_CHANGED;
+	}
+	if (result & SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED){
+		out = ms_strcat_printf(out, "%s ", "STREAMS_CHANGED");
+		result &= ~SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED;
+	}
+	if (result & SAL_MEDIA_DESCRIPTION_CRYPTO_POLICY_CHANGED){
+		out = ms_strcat_printf(out, "%s ", "CRYPTO_POLICY_CHANGED");
+		result &= ~SAL_MEDIA_DESCRIPTION_CRYPTO_POLICY_CHANGED;
+	}
+	if (result & SAL_MEDIA_DESCRIPTION_FORCE_STREAM_RECONSTRUCTION){
+		out = ms_strcat_printf(out, "%s ", "FORCE_STREAM_RECONSTRUCTION");
+		result &= ~SAL_MEDIA_DESCRIPTION_FORCE_STREAM_RECONSTRUCTION;
+	}
+	if (result){
+		ms_fatal("There are unhandled result bitmasks in sal_media_description_print_differences(), fix it");
+	}
+	if (!out) out = ms_strdup("NONE");
+	return out;
+}
+
 int sal_media_description_equals(const SalMediaDescription *md1, const SalMediaDescription *md2) {
 	int result = SAL_MEDIA_DESCRIPTION_UNCHANGED;
 	int i;
 
 	if (strcmp(md1->addr, md2->addr) != 0) result |= SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED;
+	if (md1->addr[0]!='\0' && md2->addr[0]!='\0' && ms_is_multicast(md1->addr) != ms_is_multicast(md2->addr))
+		result |= SAL_MEDIA_DESCRIPTION_NETWORK_XXXCAST_CHANGED;
 	if (md1->nb_streams != md2->nb_streams) result |= SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED;
 	if (md1->bandwidth != md2->bandwidth) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
-	for(i = 0; i < md1->nb_streams; ++i){
+
+	/* ICE */
+	if (strcmp(md1->ice_ufrag, md2->ice_ufrag) != 0) result |= SAL_MEDIA_DESCRIPTION_ICE_RESTART_DETECTED;
+	if (strcmp(md1->ice_pwd, md2->ice_pwd) != 0) result |= SAL_MEDIA_DESCRIPTION_ICE_RESTART_DETECTED;
+
+	for(i = 0; i < SAL_MEDIA_DESCRIPTION_MAX_STREAMS; ++i){
+		if (!sal_stream_description_active(&md1->streams[i]) && !sal_stream_description_active(&md2->streams[i])) continue;
 		result |= sal_stream_description_equals(&md1->streams[i], &md2->streams[i]);
 	}
 	return result;
 }
+
 static void assign_address(SalAddress** address, const char *value){
 	if (*address){
 		sal_address_destroy(*address);
@@ -613,8 +711,9 @@ void sal_auth_info_delete(SalAuthInfo* auth_info) {
 
 const char* sal_stream_type_to_string(SalStreamType type) {
 	switch (type) {
-	case SalAudio:return "audio";
-	case SalVideo:return "video";
+	case SalAudio: return "audio";
+	case SalVideo: return "video";
+	case SalText: return "text";
 	default: return "other";
 	}
 }
@@ -708,14 +807,14 @@ const char* sal_privacy_to_string(SalPrivacy privacy) {
 }
 
 static void remove_trailing_spaces(char *line){
-	int i;
+	size_t i;
 	for(i=strlen(line)-1;i>=0;--i){
 		if (isspace(line[i])) line[i]='\0';
 		else break;
 	}
 }
 
-static int line_get_value(const char *input, const char *key, char *value, size_t value_size, int *read){
+static int line_get_value(const char *input, const char *key, char *value, size_t value_size, size_t *read){
 	const char *end=strchr(input,'\n');
 	char line[256]={0};
 	char key_candidate[256];
@@ -740,7 +839,7 @@ static int line_get_value(const char *input, const char *key, char *value, size_
 }
 
 int sal_lines_get_value(const char *data, const char *key, char *value, size_t value_size){
-	int read=0;
+	size_t read=0;
 
 	do{
 		if (line_get_value(data,key,value,value_size,&read))
