@@ -25,13 +25,12 @@ static GtkWidget *linphone_gtk_create_contact_menu(GtkWidget *contact_list);
 enum{
 	FRIEND_PRESENCE_IMG,
 	FRIEND_NAME,
-	FRIEND_PRESENCE_STATUS,
 	FRIEND_ID,
 	FRIEND_CHATROOM,
 	FRIEND_SIP_ADDRESS,
-	FRIEND_ICON,
-	FRIEND_CALL,
 	FRIEND_CHAT,
+	FRIEND_CALL_BUTTON_VISIBLE,
+	FRIEND_CHAT_BUTTON_VISIBLE,
 	FRIEND_LIST_NCOL
 };
 
@@ -41,88 +40,38 @@ typedef struct _status_picture_tab_t{
 } status_picture_tab_t;
 
 status_picture_tab_t status_picture_tab[]={
-	{	LinphoneStatusOnline,		"status-green.png"	},
-	{	LinphoneStatusBusy,		"status-orange.png"		},
-	{	LinphoneStatusBeRightBack,	"status-orange.png"		},
-	{	LinphoneStatusAway,		"status-orange.png"		},
-	{	LinphoneStatusOnThePhone,	"status-orange.png"		},
-	{	LinphoneStatusOutToLunch,	"status-orange.png"		},
-	{	LinphoneStatusDoNotDisturb,	"status-red.png"	},
-	{	LinphoneStatusMoved,		"status-orange.png"	},
-	{	LinphoneStatusAltService,	"status-orange.png"	},
-	{	LinphoneStatusOffline,	"status-offline.png"		},
-	{	LinphoneStatusPending,	"status-offline.png"		},
-	{	LinphoneStatusEnd,		NULL			},
+	{ LinphoneStatusOnline       , "linphone-status-online"       },
+	{ LinphoneStatusBusy         , "linphone-status-away"         },
+	{ LinphoneStatusBeRightBack  , "linphone-status-away"         },
+	{ LinphoneStatusAway         , "linphone-status-away"         },
+	{ LinphoneStatusOnThePhone   , "linphone-status-away"         },
+	{ LinphoneStatusOutToLunch   , "linphone-status-away"         },
+	{ LinphoneStatusDoNotDisturb , "linphone-status-donotdisturb" },
+	{ LinphoneStatusMoved        , "linphone-status-away"         },
+	{ LinphoneStatusAltService   , "linphone-status-away"         },
+	{ LinphoneStatusOffline      , "linphone-status-offline"      },
+	{ LinphoneStatusPending      , "linphone-status-offline"      },
+	{ LinphoneStatusEnd          , NULL                           }
 };
 
-static GdkPixbuf *create_status_picture(LinphoneOnlineStatus ss){
+static const char *status_to_icon_name(LinphoneOnlineStatus ss) {
 	status_picture_tab_t *t=status_picture_tab;
 	while(t->img!=NULL){
-		if (ss==t->status){
-			GdkPixbuf *pixbuf;
-			pixbuf = create_pixbuf(t->img);
-			return pixbuf;
+		if (ss==t->status) {
+			return t->img;
 		}
 		++t;
 	}
-	g_error("No pixmap defined for status %i",ss);
+	g_error("No icon name defined for status %i",ss);
 	return NULL;
 }
 
-static GdkPixbuf *create_call_picture(){
-	GdkPixbuf *pixbuf;
-	pixbuf = create_pixbuf("call.png");
-	return pixbuf;
+static GtkWidget *create_status_picture(LinphoneOnlineStatus ss, GtkIconSize icon_size){
+	const char *icon_name = status_to_icon_name(ss);
+	if(icon_name) return gtk_image_new_from_icon_name(icon_name, icon_size);
+	else return NULL;
 }
 
-static GdkPixbuf *create_unread_msg(){
-	GdkPixbuf *pixbuf;
-	pixbuf = create_pixbuf("active_chat.png");
-	return pixbuf;
-}
-
-static GdkPixbuf *create_chat_picture(){
-	GdkPixbuf *pixbuf;
-	pixbuf = create_pixbuf("chat.png");
-	return pixbuf;
-}
-
-static GdkPixbuf *create_composing_unread_msg(){
-	GdkPixbuf *pixbuf;
-	pixbuf = create_pixbuf("composing_active_chat.png");
-	return pixbuf;
-}
-
-static GdkPixbuf *create_composing_chat_picture(){
-	GdkPixbuf *pixbuf;
-	pixbuf = create_pixbuf("composing_chat.png");
-	return pixbuf;
-}
-
-/*
-void linphone_gtk_set_friend_status(GtkWidget *friendlist , LinphoneFriend * fid, const gchar *url, const gchar *status, const gchar *img){
-	GtkTreeIter iter;
-	LinphoneFriend *tmp=0;
-
-	GtkTreeModel *model=gtk_tree_view_get_model(GTK_TREE_VIEW(friendlist));
-	if (gtk_tree_model_get_iter_first(model,&iter)) {
-		do{
-			gtk_tree_model_get(model,&iter,FRIEND_ID,&tmp,-1);
-			//printf("tmp=%i, fid=%i",tmp,fid);
-			if (fid==tmp) {
-				GdkPixbuf *pixbuf;
-				gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_PRESENCE_STATUS,status,-1);
-				pixbuf = create_pixbuf(img);
-				if (pixbuf)
-				{
-					gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_PRESENCE_IMG, pixbuf,-1);
-				}
-			}
-		}while(gtk_tree_model_iter_next(model,&iter));
-	}
-}
-*/
-
 gboolean linphone_gtk_friend_list_is_contact(const LinphoneAddress *addr){
 	LinphoneFriend *lf;
 	char *addr_str=linphone_address_as_string(addr);
@@ -149,7 +98,7 @@ static void linphone_gtk_set_selection_to_uri_bar(GtkTreeView *treeview){
 }
 
 void linphone_gtk_add_contact(void){
-	GtkWidget *w=linphone_gtk_create_window("contact");
+	GtkWidget *w=linphone_gtk_create_window("contact", linphone_gtk_get_main_window());
 	int presence_enabled=linphone_gtk_get_ui_config_int("use_subscribe_notify",1);
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"show_presence")),presence_enabled);
@@ -168,7 +117,7 @@ void linphone_gtk_edit_contact(GtkWidget *button){
 	if (gtk_tree_selection_get_selected (select, &model, &iter))
 	{
 		gtk_tree_model_get (model, &iter,FRIEND_ID , &lf, -1);
-		linphone_gtk_show_contact(lf);
+		linphone_gtk_show_contact(lf, w);
 	}
 }
 
@@ -233,32 +182,55 @@ static void linphone_gtk_call_selected(GtkTreeView *treeview){
 					"start_call"));
 }
 
-void linphone_gtk_friend_list_update_chat_picture(){
-	GtkTreeIter iter;
-	GtkWidget *w = linphone_gtk_get_main_window();
-	GtkWidget *friendlist=linphone_gtk_get_widget(w,"contact_list");
-	GtkTreeModel *model=gtk_tree_view_get_model(GTK_TREE_VIEW(friendlist));
+void linphone_gtk_friend_list_update_button_display(GtkTreeView *friendlist){
+	GtkTreeIter iter, selected_iter;
+	GtkTreeModel *model=gtk_tree_view_get_model(friendlist);
+	GtkTreeSelection *select=gtk_tree_view_get_selection(friendlist);
 	LinphoneChatRoom *cr=NULL;
-	bool_t is_composing;
+	gboolean is_composing;
 	int nbmsg=0;
+	GtkTreePath *selected_path = NULL;
+	GtkTreePath *hovered_row = (GtkTreePath *)g_object_get_data(G_OBJECT(friendlist), "hovered_row");
+	
+	if (gtk_tree_selection_get_selected(select, &model, &selected_iter)){
+		selected_path = gtk_tree_model_get_path(model, &selected_iter);
+	}
+	
 	if (gtk_tree_model_get_iter_first(model,&iter)) {
 		do{
+			const char *icon_name = NULL;
+			gboolean show_chat_button = FALSE;
+			gboolean show_call_button = FALSE;
+			GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
+			
 			gtk_tree_model_get (model, &iter,FRIEND_CHATROOM , &cr, -1);
 			nbmsg=linphone_chat_room_get_unread_messages_count(cr);
 			is_composing=linphone_chat_room_is_remote_composing(cr);
 			if(nbmsg != 0){
-				if (is_composing == TRUE)
-					gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_composing_unread_msg(),-1);
-				else
-					gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_unread_msg(),-1);
+				if (is_composing) icon_name = "linphone-chat-new-message-and-writing";
+				else icon_name = "linphone-chat-new-message";
+				show_chat_button = TRUE;
 			} else {
-				if (is_composing == TRUE)
-					gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_composing_chat_picture(),-1);
-				else
-					gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,create_chat_picture(),-1);
+				if (is_composing) {
+					icon_name = "linphone-chat-writing";
+					show_chat_button = TRUE;
+				} else {
+					icon_name = "linphone-chat-nothing";
+				}
 			}
+			if ((selected_path && gtk_tree_path_compare(path, selected_path) == 0) 
+					|| (hovered_row && gtk_tree_path_compare(path, hovered_row) == 0)){
+				show_chat_button = TRUE;
+				show_call_button = TRUE;
+			}
+			gtk_list_store_set(GTK_LIST_STORE(model),&iter,FRIEND_CHAT,icon_name,
+						FRIEND_CHAT_BUTTON_VISIBLE, show_chat_button, -1);
+			gtk_list_store_set(GTK_LIST_STORE(model), &iter, FRIEND_CALL_BUTTON_VISIBLE, show_call_button, -1);
+			
+			gtk_tree_path_free(path);
 		}while(gtk_tree_model_iter_next(model,&iter));
 	}
+	if (selected_path) gtk_tree_path_free(selected_path);
 }
 
 static gboolean grab_focus(GtkWidget *w){
@@ -269,9 +241,7 @@ static gboolean grab_focus(GtkWidget *w){
 void linphone_gtk_friend_list_set_active_address(const LinphoneAddress *addr){
 	GtkWidget *w=linphone_gtk_get_main_window();
 	GtkWidget *friendlist=linphone_gtk_get_widget(w,"contact_list");
-	LinphoneAddress *old_addr=(LinphoneAddress*)g_object_get_data(G_OBJECT(friendlist),"from");
-	g_object_set_data(G_OBJECT(friendlist),"from", addr ? linphone_address_clone(addr) : NULL);
-	if (old_addr) linphone_address_unref(old_addr);
+	g_object_set_data_full(G_OBJECT(friendlist),"from", addr ? linphone_address_clone(addr) : NULL, (GDestroyNotify)linphone_address_destroy);
 }
 
 const LinphoneAddress *linphone_gtk_friend_list_get_active_address(void){
@@ -290,20 +260,12 @@ void linphone_gtk_friend_list_set_chat_conversation(const LinphoneAddress *la){
 	LinphoneFriend *lf=NULL;
 	LinphoneChatRoom *cr=NULL;
 	GtkNotebook *notebook=(GtkNotebook *)linphone_gtk_get_widget(w,"viewswitch");
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(friendlist));
 
 	lf=linphone_core_find_friend(linphone_gtk_get_core(),la);
 	if(lf==NULL){
 		cr=linphone_gtk_create_chatroom(la);
 		linphone_gtk_friend_list_set_active_address(la);
-		if(chat_view==NULL){
-			chat_view=linphone_gtk_init_chatroom(cr,la);
-			g_object_set_data(G_OBJECT(friendlist),"chatview",(gpointer)chat_view);
-		} else {
-			linphone_gtk_load_chatroom(cr,la,chat_view);
-		}
-		gtk_notebook_set_current_page(notebook,gtk_notebook_page_num(notebook,chat_view));
-		linphone_gtk_friend_list_update_chat_picture();
-		g_idle_add((GSourceFunc)grab_focus,linphone_gtk_get_widget(chat_view,"text_entry"));
 	} else {
 		store=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(friendlist)));
 		if (gtk_tree_model_get_iter_first(model,&iter)) {
@@ -318,20 +280,22 @@ void linphone_gtk_friend_list_set_chat_conversation(const LinphoneAddress *la){
 						gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
 					}
 					linphone_gtk_friend_list_set_active_address(uri);
-					if(chat_view==NULL){
-						chat_view=linphone_gtk_init_chatroom(cr,uri);
-						g_object_set_data(G_OBJECT(friendlist),"chatview",(gpointer)chat_view);
-					} else {
-						linphone_gtk_load_chatroom(cr,uri,chat_view);
-					}
-					gtk_notebook_set_current_page(notebook,gtk_notebook_page_num(notebook,chat_view));
-					linphone_gtk_friend_list_update_chat_picture();
-					g_idle_add((GSourceFunc)grab_focus,linphone_gtk_get_widget(chat_view,"text_entry"));
+					gtk_tree_selection_select_iter(selection, &iter);
 					break;
 				}
 			}while(gtk_tree_model_iter_next(model,&iter));
 		}
 	}
+	if(cr) {
+		if(chat_view == NULL){
+			chat_view=linphone_gtk_init_chatroom(cr,la);
+			g_object_set_data(G_OBJECT(friendlist),"chatview",(gpointer)chat_view);
+		} else {
+			linphone_gtk_load_chatroom(cr,la,chat_view);
+		}
+		gtk_notebook_set_current_page(notebook,gtk_notebook_page_num(notebook,chat_view));
+		g_idle_add((GSourceFunc)grab_focus,linphone_gtk_get_widget(chat_view,"text_entry"));
+	}
 }
 
 void linphone_gtk_notebook_tab_select(GtkNotebook *notebook,GtkWidget *page,guint page_num, gpointer data){
@@ -375,7 +339,7 @@ void linphone_gtk_chat_selected(GtkWidget *item){
 			cr=linphone_gtk_create_chatroom(uri);
 			gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
 		}
-		page=(GtkWidget*)g_object_get_data(G_OBJECT(friendlist),"chatview");
+		page=GTK_WIDGET(g_object_get_data(G_OBJECT(friendlist),"chatview"));
 		linphone_gtk_friend_list_set_active_address(uri);
 		if(page==NULL){
 			page=linphone_gtk_init_chatroom(cr,uri);
@@ -385,29 +349,25 @@ void linphone_gtk_chat_selected(GtkWidget *item){
 		}
 		linphone_chat_room_mark_as_read(cr);
 		gtk_notebook_set_current_page(notebook,gtk_notebook_page_num(notebook,page));
-		linphone_gtk_friend_list_update_chat_picture();
 		g_idle_add((GSourceFunc)grab_focus,linphone_gtk_get_widget(page,"text_entry"));
 	}
 }
 
-void linphone_gtk_contact_activated(GtkTreeView     *treeview,
-                                    GtkTreePath     *path,
-                                    GtkTreeViewColumn *column,
-                                        gpointer         user_data)
-{
-	//linphone_gtk_call_selected(treeview);
-}
-
-void linphone_gtk_contact_clicked(GtkTreeView     *treeview){
-	linphone_gtk_set_selection_to_uri_bar(treeview);
-	if(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(treeview),"numcol"))==1){
-		linphone_gtk_call_selected(treeview);
+void linphone_gtk_contact_clicked(GtkTreeSelection *selection){
+	GtkTreeView *friendlist = gtk_tree_selection_get_tree_view(selection);
+	GtkWidget *mw = linphone_gtk_get_main_window();
+	GtkWidget *edit_button = linphone_gtk_get_widget(mw, "edit_button");
+	GtkWidget *remove_button = linphone_gtk_get_widget(mw, "remove_button");
+	
+	linphone_gtk_set_selection_to_uri_bar(friendlist);
+	linphone_gtk_friend_list_update_button_display(friendlist);
+	if(gtk_tree_selection_get_selected(selection, NULL, NULL)) {
+		gtk_widget_set_sensitive(edit_button, TRUE);
+		gtk_widget_set_sensitive(remove_button, TRUE);
 	} else {
-		if(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(treeview),"numcol"))==2){
-			linphone_gtk_chat_selected(GTK_WIDGET(treeview));
-		}
+		gtk_widget_set_sensitive(edit_button, FALSE);
+		gtk_widget_set_sensitive(remove_button, FALSE);
 	}
-	g_object_set_data(G_OBJECT(treeview),"numcol",GINT_TO_POINTER(0));
 }
 
 
@@ -426,17 +386,14 @@ void linphone_gtk_remove_button_clicked(GtkWidget *button){
 static GtkWidget * create_presence_menu(){
 	GtkWidget *menu=gtk_menu_new();
 	GtkWidget *menu_item;
-	GdkPixbuf *pbuf;
 	status_picture_tab_t *t;
 	for(t=status_picture_tab;t->img!=NULL;++t){
 		if (t->status==LinphoneStatusPending){
 			continue;
 		}
 		menu_item=gtk_image_menu_item_new_with_label(linphone_online_status_to_string(t->status));
-		pbuf=create_status_picture(t->status);
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),
-						gtk_image_new_from_pixbuf(pbuf));
-		g_object_unref(G_OBJECT(pbuf));
+				gtk_image_new_from_icon_name(t->img, GTK_ICON_SIZE_LARGE_TOOLBAR));
 		gtk_widget_show(menu_item);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
 		g_signal_connect_swapped(G_OBJECT(menu_item),"activate",(GCallback)linphone_gtk_set_my_presence,GINT_TO_POINTER(t->status));
@@ -446,10 +403,9 @@ static GtkWidget * create_presence_menu(){
 
 void linphone_gtk_set_my_presence(LinphoneOnlineStatus ss){
 	GtkWidget *button=linphone_gtk_get_widget(linphone_gtk_get_main_window(),"presence_button");
-	GdkPixbuf *pbuf=create_status_picture(ss);
-	GtkWidget *image=gtk_image_new_from_pixbuf(pbuf);
+	GtkWidget *image=create_status_picture(ss, GTK_ICON_SIZE_LARGE_TOOLBAR);
 	GtkWidget *menu;
-	g_object_unref(G_OBJECT(pbuf));
+
 	gtk_widget_set_tooltip_text(button,linphone_online_status_to_string(ss));
 	gtk_button_set_image(GTK_BUTTON(button),image);
 	/*prepare menu*/
@@ -471,6 +427,7 @@ void linphone_gtk_my_presence_clicked(GtkWidget *button){
 }
 
 static void icon_press_handler(GtkEntry *entry){
+	GtkWidget *w = gtk_widget_get_toplevel(GTK_WIDGET(entry));
 	const char *text=gtk_entry_get_text(entry);
 	if (text && strlen(text)>0){
 		LinphoneAddress *addr;
@@ -487,57 +444,38 @@ static void icon_press_handler(GtkEntry *entry){
 			lf=linphone_friend_new();
 		if (lf!=NULL){
 			linphone_friend_set_address(lf,addr);
-			linphone_gtk_show_contact(lf);
+			linphone_gtk_show_contact(lf, w);
 		}
 		linphone_address_destroy(addr);
 	}
 }
 
 static void update_star(GtkEntry *entry, gboolean is_known){
-	GdkPixbuf *active,*starred,*unstarred;
-	active=gtk_entry_get_icon_pixbuf(entry,GTK_ENTRY_ICON_SECONDARY);
-	starred=g_object_get_data(G_OBJECT(entry),"starred_icon");
-	unstarred=g_object_get_data(G_OBJECT(entry),"unstarred_icon");
-	if (is_known && (active==unstarred)){
-		gtk_entry_set_icon_from_pixbuf(entry,GTK_ENTRY_ICON_SECONDARY,starred);
+	if (is_known){
+		gtk_entry_set_icon_from_icon_name(entry,GTK_ENTRY_ICON_SECONDARY,NULL);
+		gtk_entry_set_icon_sensitive(GTK_ENTRY(entry),GTK_ENTRY_ICON_SECONDARY,FALSE);
 		gtk_entry_set_icon_tooltip_text(GTK_ENTRY(entry),GTK_ENTRY_ICON_SECONDARY,NULL);
-	}else if ((!is_known) && (active==starred)){
+	}else{
+		gtk_entry_set_icon_from_icon_name(entry,GTK_ENTRY_ICON_SECONDARY,"linphone-contact-add");
+		gtk_entry_set_icon_sensitive(GTK_ENTRY(entry),GTK_ENTRY_ICON_SECONDARY,TRUE);
 		gtk_entry_set_icon_tooltip_text(GTK_ENTRY(entry),GTK_ENTRY_ICON_SECONDARY,_("Add to addressbook"));
-		gtk_entry_set_icon_from_pixbuf(entry,GTK_ENTRY_ICON_SECONDARY,unstarred);
 	}
 }
 
 static void check_contact(GtkEditable *editable, LinphoneCore *lc){
-	char *tmp=gtk_editable_get_chars(editable,0,-1);
-	if (tmp!=NULL){
-		if (strlen(tmp)>0){
-			LinphoneAddress *addr=linphone_core_interpret_url(lc,tmp);
-			if (addr){
-				char *uri=linphone_address_as_string_uri_only(addr);
-				LinphoneFriend *lf=linphone_core_get_friend_by_address(lc,uri);
-				ms_free(uri);
-				linphone_address_destroy(addr);
-				if (lf) {
-					update_star(GTK_ENTRY(editable),TRUE);
-					g_free(tmp);
-					return;
-				}
-			}
+	bool_t known = TRUE;
+	char *tmp = gtk_editable_get_chars(editable, 0, -1);
+	if (tmp != NULL) {
+		if (strlen(tmp) > 0) {
+			known = linphone_gtk_is_friend(lc, tmp);
 		}
 		g_free(tmp);
 	}
-	update_star(GTK_ENTRY(editable),FALSE);
+	update_star(GTK_ENTRY(editable), known);
 }
 
 static void linphone_gtk_init_bookmark_icon(void){
-	GtkWidget *mw=linphone_gtk_get_main_window();
-	GtkWidget *entry=linphone_gtk_get_widget(mw,"uribar");
-	GdkPixbuf *pbuf=create_pixbuf("contact_unstarred.png");
-	gtk_entry_set_icon_from_pixbuf(GTK_ENTRY(entry),GTK_ENTRY_ICON_SECONDARY,pbuf);
-	g_object_set_data_full(G_OBJECT(entry),"unstarred_icon",pbuf,g_object_unref);
-	pbuf=create_pixbuf("contact_starred.png");
-	g_object_set_data_full(G_OBJECT(entry),"starred_icon",pbuf,g_object_unref);
-	gtk_entry_set_icon_activatable(GTK_ENTRY(entry),GTK_ENTRY_ICON_SECONDARY,TRUE);
+	GtkWidget *entry = linphone_gtk_get_widget(linphone_gtk_get_main_window(), "uribar");
 	g_signal_connect(G_OBJECT(entry),"icon-release",(GCallback)icon_press_handler,NULL);
 	g_signal_connect(G_OBJECT(GTK_EDITABLE(entry)),"changed",(GCallback)check_contact,linphone_gtk_get_core());
 }
@@ -579,7 +517,7 @@ static gint friend_sort(GtkTreeModel *model, GtkTreeIter *a,GtkTreeIter *b,gpoin
 	return ret;
 }
 
-static void on_name_column_clicked(GtkTreeModel *model){
+void linphone_gtk_friend_list_on_name_column_clicked(GtkTreeModel *model){
 	GtkSortType st;
 	gint column;
 
@@ -645,7 +583,8 @@ static MSList *sort_friend_list(const MSList *friends){
 	return ret;
 }
 
-static void on_presence_column_clicked(GtkTreeModel *model){
+#if 0
+void linphone_gtk_friend_list_on_presence_column_clicked(GtkTreeModel *model){
 	GtkSortType st;
 	gint column;
 
@@ -656,87 +595,19 @@ static void on_presence_column_clicked(GtkTreeModel *model){
 	}else st=GTK_SORT_ASCENDING;
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model),FRIEND_ID,st);
 }
-
-void create_button(){
-	GtkWidget *main_window = linphone_gtk_get_main_window ();
-	GtkWidget *button_add = linphone_gtk_get_widget(main_window,"add_button");
-	GtkWidget *image;
-
-	image=gtk_image_new_from_stock(GTK_STOCK_ADD,GTK_ICON_SIZE_MENU);
-	gtk_container_add (GTK_CONTAINER (button_add), image);
-}
-
-static void linphone_gtk_friend_list_init(GtkWidget *friendlist){
-	GtkListStore *store;
-	GtkCellRenderer *renderer;
-	GtkTreeViewColumn *column;
-	GtkTreeSelection *select;
-
-	linphone_gtk_init_bookmark_icon();
-
-	store = gtk_list_store_new(FRIEND_LIST_NCOL,GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING,  G_TYPE_POINTER,
-								G_TYPE_POINTER, G_TYPE_STRING, GDK_TYPE_PIXBUF, GDK_TYPE_PIXBUF, GDK_TYPE_PIXBUF);
-
-	gtk_tree_view_set_model(GTK_TREE_VIEW(friendlist),GTK_TREE_MODEL(store));
-	g_object_unref(G_OBJECT(store));
-
-	/* Tree specification*/
-	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(friendlist),FALSE);
-	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(friendlist),friend_search_func,NULL,NULL);
-	gtk_tree_view_set_search_column(GTK_TREE_VIEW(friendlist),FRIEND_NAME);
-	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store),FRIEND_NAME,friend_sort,NULL,NULL);
-
-	/*Name and presence column*/
-	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Presence status"),
-                                                   renderer,
-                                                   "text", FRIEND_PRESENCE_STATUS,
-                                                   NULL);
-	g_object_set (G_OBJECT(column), "resizable", TRUE, NULL);
-	g_signal_connect_swapped(G_OBJECT(column),"clicked",(GCallback)on_presence_column_clicked,GTK_TREE_MODEL(store));
-	gtk_tree_view_column_set_clickable(column,TRUE);
-	gtk_tree_view_column_set_visible(column,linphone_gtk_get_ui_config_int("friendlist_status",1));
-	gtk_tree_view_column_set_min_width(column,50);
-
-	renderer = gtk_cell_renderer_pixbuf_new();
-	gtk_tree_view_column_pack_start(column,renderer,TRUE);
-	gtk_tree_view_column_add_attribute  (column,renderer,
-                                                         "pixbuf",
-                                                         FRIEND_PRESENCE_IMG);
-	gtk_tree_view_append_column (GTK_TREE_VIEW (friendlist), column);
-
-	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Name"),
-                                                   renderer,
-                                                   "text", FRIEND_NAME,NULL);
-	g_object_set (G_OBJECT(column), "resizable", TRUE, NULL);
-	g_signal_connect_swapped(G_OBJECT(column),"clicked",(GCallback)on_name_column_clicked,GTK_TREE_MODEL(store));
-	gtk_tree_view_column_set_clickable(column,TRUE);
-	gtk_tree_view_column_set_expand(column,TRUE);
-	gtk_tree_view_column_set_max_width(column,60);
-	gtk_tree_view_append_column (GTK_TREE_VIEW (friendlist), column);
-
-	/* Call column*/
-	renderer = gtk_cell_renderer_pixbuf_new();
-	column = gtk_tree_view_column_new_with_attributes (_("Call"),renderer,"pixbuf",FRIEND_CALL,NULL);
-	gtk_tree_view_append_column (GTK_TREE_VIEW (friendlist), column);
-
-	/* Chat column*/
-	renderer = gtk_cell_renderer_pixbuf_new();
-	column = gtk_tree_view_column_new_with_attributes (_("Chat"),renderer,"pixbuf",FRIEND_CHAT,NULL);
-	gtk_tree_view_append_column (GTK_TREE_VIEW (friendlist), column);
-
-	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (friendlist));
-	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
-
-	gtk_tree_view_set_grid_lines(GTK_TREE_VIEW(friendlist),GTK_TREE_VIEW_GRID_LINES_NONE);
-#if GTK_CHECK_VERSION(2,12,0)
-	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(friendlist),FRIEND_SIP_ADDRESS);
 #endif
 
-	gtk_widget_set_size_request(friendlist,200,120);
-	/*gtk_combo_box_set_active(GTK_COMBO_BOX(linphone_gtk_get_widget(
-					gtk_widget_get_toplevel(friendlist),"show_category")),0);*/
+static void linphone_gtk_friend_list_init(GtkWidget *friendlist){
+	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(friendlist));
+	GtkTreeSelection *select = gtk_tree_view_get_selection (GTK_TREE_VIEW (friendlist));
+
+	linphone_gtk_init_bookmark_icon();
+	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(friendlist),friend_search_func,NULL,NULL);
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store),FRIEND_NAME,friend_sort,NULL,NULL);
+	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
+	g_signal_connect(G_OBJECT(select), "changed", G_CALLBACK(linphone_gtk_contact_clicked), NULL);
+	
+	g_object_set_data(G_OBJECT(friendlist), "friendlist_initialized", (gpointer)TRUE);
 }
 
 void linphone_gtk_show_directory_search(void){
@@ -799,26 +670,18 @@ void linphone_gtk_show_friends(void){
 	GtkListStore *store=NULL;
 	GtkTreeIter iter;
 	const MSList *itf;
-	//GtkWidget *filter=linphone_gtk_get_widget(mw,"search_bar");
 	LinphoneCore *core=linphone_gtk_get_core();
-	//const gchar *search=NULL;
-	//gboolean lookup=FALSE;
 	MSList *sorted;
 	LinphoneChatRoom *cr=NULL;
 
 	linphone_gtk_show_directory_search();
-	if (gtk_tree_view_get_model(GTK_TREE_VIEW(friendlist))==NULL){
+	if (!g_object_get_data(G_OBJECT(friendlist), "friendlist_initialized")) {
 		linphone_gtk_friend_list_init(friendlist);
 	}
 
 	store=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(friendlist)));
 	gtk_list_store_clear(store);
 
-	//search=gtk_entry_get_text(GTK_ENTRY(filter));
-	//if (search==NULL || search[0]=='\0')
-	//	lookup=FALSE;
-	//else lookup=TRUE;
-
 	sorted=sort_friend_list(linphone_core_get_friend_list(core));
 
 	for(itf=sorted;itf!=NULL;itf=ms_list_next(itf)){
@@ -828,15 +691,8 @@ void linphone_gtk_show_friends(void){
 		const char *name=linphone_address_get_display_name(f_uri);
 		const char *display=name;
 		char *escaped=NULL;
-		//char buf[26]={0};
 		int nbmsg=0;
 
-		/*if (lookup){
-			if (strstr(uri,search)==NULL){
-				ms_free(uri);
-				continue;
-			}
-		}*/
 		//BuddyInfo *bi;
 		gboolean send_subscribe=linphone_friend_get_send_subscribe(lf);
 		if (name==NULL || name[0]=='\0') {
@@ -844,33 +700,24 @@ void linphone_gtk_show_friends(void){
 		}
 		gtk_list_store_append(store,&iter);
 		gtk_list_store_set(store,&iter,FRIEND_NAME, display,FRIEND_ID,lf,
-			    FRIEND_PRESENCE_IMG, send_subscribe ? create_status_picture(linphone_friend_get_status(lf)) : NULL,
-				FRIEND_CHAT,create_chat_picture(),FRIEND_CALL,create_call_picture(),-1);
+				FRIEND_PRESENCE_IMG, send_subscribe ? status_to_icon_name(linphone_friend_get_status(lf)) : NULL,
+				FRIEND_CHAT,"linphone-chat-nothing", -1);
 		cr=linphone_gtk_create_chatroom(f_uri);
 		gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
 		nbmsg=linphone_chat_room_get_unread_messages_count(cr);
 		if(nbmsg != 0){
-			gtk_list_store_set(store,&iter,FRIEND_CHAT,create_unread_msg(),-1);
+			gtk_list_store_set(store,&iter,FRIEND_CHAT,"linphone-chat-new-message",-1);
 		}
 		escaped=g_markup_escape_text(uri,-1);
 		gtk_list_store_set(store,&iter,FRIEND_SIP_ADDRESS,escaped,-1);
 		g_free(escaped);
-		//bi=linphone_friend_get_info(lf);
-		/*if (bi!=NULL && bi->image_data!=NULL){
-			GdkPixbuf *pbuf=
-				_gdk_pixbuf_new_from_memory_at_scale(bi->image_data,bi->image_length,-1,40,TRUE);
-			if (pbuf) {
-				//gtk_list_store_set(store,&iter,FRIEND_ICON,pbuf,-1);
-				g_object_unref(G_OBJECT(pbuf));
-			}
-		}*/
 		ms_free(uri);
 	}
 	ms_list_free(sorted);
 }
 
-void linphone_gtk_show_contact(LinphoneFriend *lf){
-	GtkWidget *w=linphone_gtk_create_window("contact");
+void linphone_gtk_show_contact(LinphoneFriend *lf, GtkWidget *parent){
+	GtkWidget *w=linphone_gtk_create_window("contact", parent);
 	char *uri;
 	const char *name;
 	const LinphoneAddress *f_uri=linphone_friend_get_address(lf);
@@ -888,6 +735,7 @@ void linphone_gtk_show_contact(LinphoneFriend *lf){
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"allow_presence")),
 					linphone_friend_get_inc_subscribe_policy(lf)==LinphoneSPAccept);
 	g_object_set_data(G_OBJECT(w),"friend_ref",(gpointer)lf);
+	
 	gtk_widget_show(w);
 }
 
@@ -948,11 +796,10 @@ void linphone_gtk_contact_ok(GtkWidget *button){
 static GtkWidget *linphone_gtk_create_contact_menu(GtkWidget *contact_list){
 	GtkWidget *menu=gtk_menu_new();
 	GtkWidget *menu_item;
-	gchar *call_label=NULL;
-	gchar *text_label=NULL;
 	gchar *edit_label=NULL;
 	gchar *delete_label=NULL;
 	gchar *delete_hist_label=NULL;
+	gchar *add_contact_label=NULL;
 	gchar *name=NULL;
 	GtkTreeSelection *select;
 	GtkTreeIter iter;
@@ -961,6 +808,7 @@ static GtkWidget *linphone_gtk_create_contact_menu(GtkWidget *contact_list){
 	LinphoneCore *lc=linphone_gtk_get_core();
 	LinphoneProxyConfig *cfg=NULL;
 	SipSetupContext * ssc=NULL;
+	bool_t show_menu_separator=FALSE;
 
 	linphone_core_get_default_proxy(lc,&cfg);
 	if (cfg){
@@ -969,36 +817,18 @@ static GtkWidget *linphone_gtk_create_contact_menu(GtkWidget *contact_list){
 
 	g_signal_connect(G_OBJECT(menu), "selection-done", G_CALLBACK (gtk_widget_destroy), NULL);
 	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(contact_list));
+	add_contact_label=g_strdup_printf(_("Add a new contact"));
 	if (gtk_tree_selection_get_selected (select, &model, &iter)){
 		gtk_tree_model_get(model, &iter,FRIEND_NAME , &name, -1);
-		call_label=g_strdup_printf(_("Call %s"),name);
-		text_label=g_strdup_printf(_("Send text to %s"),name);
 		edit_label=g_strdup_printf(_("Edit contact '%s'"),name);
 		delete_label=g_strdup_printf(_("Delete contact '%s'"),name);
 		delete_hist_label=g_strdup_printf(_("Delete chat history of '%s'"),name);
 		g_free(name);
-	}
-	if (call_label){
-		menu_item=gtk_image_menu_item_new_with_label(call_label);
-		image=gtk_image_new_from_stock(GTK_STOCK_NETWORK,GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),image);
-		gtk_widget_show(image);
-		gtk_widget_show(menu_item);
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
-		g_signal_connect_swapped(G_OBJECT(menu_item),"activate",(GCallback)linphone_gtk_call_selected,contact_list);
-	}
-	if (text_label){
-		menu_item=gtk_image_menu_item_new_with_label(text_label);
-		image=gtk_image_new_from_stock(GTK_STOCK_NETWORK,GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),image);
-		gtk_widget_show(image);
-		gtk_widget_show(menu_item);
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
-		g_signal_connect_swapped(G_OBJECT(menu_item),"activate",(GCallback)linphone_gtk_chat_selected,contact_list);
+		show_menu_separator=TRUE;
 	}
 	if (edit_label){
 		menu_item=gtk_image_menu_item_new_with_label(edit_label);
-		image=gtk_image_new_from_stock(GTK_STOCK_EDIT,GTK_ICON_SIZE_MENU);
+		image=gtk_image_new_from_icon_name("linphone-edit",GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),image);
 		gtk_widget_show(image);
 		gtk_widget_show(menu_item);
@@ -1007,7 +837,7 @@ static GtkWidget *linphone_gtk_create_contact_menu(GtkWidget *contact_list){
 	}
 	if (delete_label){
 		menu_item=gtk_image_menu_item_new_with_label(delete_label);
-		image=gtk_image_new_from_stock(GTK_STOCK_DELETE,GTK_ICON_SIZE_MENU);
+		image=gtk_image_new_from_icon_name("linphone-delete",GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),image);
 		gtk_widget_show(image);
 		gtk_widget_show(menu_item);
@@ -1016,8 +846,11 @@ static GtkWidget *linphone_gtk_create_contact_menu(GtkWidget *contact_list){
 	}
 
 	if (delete_hist_label){
+		GtkWidget *menu_item_separator=gtk_separator_menu_item_new();
+		gtk_widget_show(menu_item_separator);
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item_separator);
 		menu_item=gtk_image_menu_item_new_with_label(delete_hist_label);
-		image=gtk_image_new_from_stock(GTK_STOCK_CLEAR,GTK_ICON_SIZE_MENU);
+		image=gtk_image_new_from_icon_name("linphone-delete",GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),image);
 		gtk_widget_show(image);
 		gtk_widget_show(menu_item);
@@ -1029,24 +862,31 @@ static GtkWidget *linphone_gtk_create_contact_menu(GtkWidget *contact_list){
 		gchar *tmp=g_strdup_printf(_("Add new contact from %s directory"),linphone_proxy_config_get_domain(cfg));
 		menu_item=gtk_image_menu_item_new_with_label(tmp);
 		g_free(tmp);
-		image=gtk_image_new_from_stock(GTK_STOCK_ADD,GTK_ICON_SIZE_MENU);
+		image=gtk_image_new_from_icon_name("linphone-contact-add",GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),image);
 		gtk_widget_show(image);
 		gtk_widget_show(menu_item);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
 		g_signal_connect_swapped(G_OBJECT(menu_item),"activate",(GCallback)linphone_gtk_show_buddy_lookup_window,ssc);
-		gtk_widget_show(menu);
 	}
 
-	menu_item=gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD,NULL);
+	if (show_menu_separator) {
+		GtkWidget *menu_item_separator=gtk_separator_menu_item_new();
+		gtk_widget_show(menu_item_separator);
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item_separator);
+	}
+
+	menu_item=gtk_image_menu_item_new_with_label(add_contact_label);
+	image=gtk_image_new_from_icon_name("linphone-contact-add",GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),image);
+	gtk_widget_show(image);
 	gtk_widget_show(menu_item);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
 	g_signal_connect_swapped(G_OBJECT(menu_item),"activate",(GCallback)linphone_gtk_add_contact,contact_list);
 	gtk_widget_show(menu);
 	gtk_menu_attach_to_widget (GTK_MENU (menu), contact_list, NULL);
 
-	if (call_label) g_free(call_label);
-	if (text_label) g_free(text_label);
+	g_free(add_contact_label);
 	if (edit_label) g_free(edit_label);
 	if (delete_label) g_free(delete_label);
 	return menu;
@@ -1059,61 +899,88 @@ gboolean linphone_gtk_popup_contact_menu(GtkWidget *list, GdkEventButton *event)
 	return TRUE;
 }
 
-gint get_col_number_from_tree_view_column (GtkTreeViewColumn *col){
-	GList *cols;
-	gint   num;
-	g_return_val_if_fail ( col != NULL, -1 );
-	g_return_val_if_fail ( col->tree_view != NULL, -1 );
-	cols = gtk_tree_view_get_columns(GTK_TREE_VIEW(col->tree_view));
-	num = g_list_index(cols, (gpointer) col);
-	g_list_free(cols);
-
-	return num;
+static int get_column_index(GtkTreeView *friendlist, const GtkTreeViewColumn *column) {
+	GList *columns = gtk_tree_view_get_columns(friendlist);
+	int i = g_list_index(columns, column);
+	g_list_free(columns);
+	return i;
 }
 
-static gint tree_view_get_cell_from_pos(GtkTreeView *view, guint x, guint y){
-	GtkTreeViewColumn *col = NULL;
-	GList *node, *columns;
-	gint colx = 0;
+static void select_row(GtkTreeView *treeview, GtkTreePath *path) {
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
+	gtk_tree_selection_select_path(selection, path);
+}
+
+gboolean linphone_gtk_contact_list_button_pressed(GtkTreeView *friendlist, GdkEventButton *event){
+	/* Ignore double-clicks and triple-clicks */
+	gboolean ret = FALSE;
+	int x_bin, y_bin;
 	GtkTreePath *path;
-	GtkTreeViewDropPosition pos;
-
-	g_return_val_if_fail ( view != NULL, 0 );
-	columns = gtk_tree_view_get_columns(view);
-
-	gtk_tree_view_get_dest_row_at_pos(view,x,y,&path,&pos);
-	if(path != NULL){
-		for (node = columns;  node != NULL && col == NULL;  node = node->next){
-			GtkTreeViewColumn *checkcol = (GtkTreeViewColumn*) node->data;
-			if (x >= colx  &&  x < (colx + checkcol->width)){
-				col = checkcol;
-				return get_col_number_from_tree_view_column(col);
-			} else {
-				colx += checkcol->width;
+	GtkTreeViewColumn *column;
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(friendlist);
+	
+	gtk_tree_view_convert_widget_to_bin_window_coords(friendlist, event->x, event->y, &x_bin, &y_bin);
+	gtk_tree_view_get_path_at_pos(friendlist, x_bin, y_bin, &path, &column, NULL, NULL);
+	
+	if (event->button == 3 && event->type == GDK_BUTTON_PRESS) {
+		if(path) gtk_tree_selection_select_path(selection, path);
+		ret = linphone_gtk_popup_contact_menu(GTK_WIDGET(friendlist), event);
+	} else if(event->button == 1 && event->type == GDK_BUTTON_PRESS){
+		if(path && column) {
+			int numcol = get_column_index(friendlist, column);
+			if(numcol == 2) {
+				select_row(friendlist, path);
+				linphone_gtk_call_selected(friendlist);
+				ret = TRUE;
+			} else if(numcol == 3) {
+				select_row(friendlist, path);
+				linphone_gtk_chat_selected(GTK_WIDGET(friendlist));
+				ret = TRUE;
 			}
 		}
 	}
-	g_list_free(columns);
-	return 0;
-}
-
-gboolean linphone_gtk_contact_list_button_pressed(GtkWidget *widget, GdkEventButton *event){
-	/* Ignore double-clicks and triple-clicks */
-	if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
-	{
-		return linphone_gtk_popup_contact_menu(widget, event);
-	} else if(event->button == 1 && event->type == GDK_BUTTON_PRESS){
-		gint numcol = tree_view_get_cell_from_pos(GTK_TREE_VIEW(widget),event->x,event->y);
-		if(numcol==2){
-			g_object_set_data(G_OBJECT(widget),"numcol",GINT_TO_POINTER(1));
-		} else if(numcol==3){
-			g_object_set_data(G_OBJECT(widget),"numcol",GINT_TO_POINTER(2));
-		}
-	}
-	return FALSE;
+	if(path) gtk_tree_path_free(path);
+	return ret;
 }
 
 void linphone_gtk_buddy_info_updated(LinphoneCore *lc, LinphoneFriend *lf){
 	/*refresh the entire list*/
 	linphone_gtk_show_friends();
 }
+
+static gboolean update_hovered_row_path(GtkTreeView *friendlist, int x_window, int y_window) {
+	int x_bin, y_bin;
+	GtkTreePath *new_path;
+	GtkTreePath *old_path = (GtkTreePath *)g_object_get_data(G_OBJECT(friendlist), "hovered_row");
+	gtk_tree_view_convert_widget_to_bin_window_coords(friendlist, x_window, y_window, &x_bin, &y_bin);
+	gtk_tree_view_get_path_at_pos(friendlist, x_bin, y_bin, &new_path, NULL, NULL, NULL);
+	if((new_path == NULL && old_path == NULL) || (new_path && old_path && gtk_tree_path_compare(new_path, old_path) == 0)) {
+		if(new_path) gtk_tree_path_free(new_path);
+		return FALSE;
+	} else {
+		g_object_set_data_full(G_OBJECT(friendlist), "hovered_row", new_path, (GDestroyNotify)gtk_tree_path_free);
+		return TRUE;
+	}
+}
+
+gboolean linphone_gtk_friend_list_enter_event_handler(GtkTreeView *friendlist, GdkEventCrossing *event) {
+	gboolean path_has_changed = update_hovered_row_path(friendlist, event->x, event->y);
+	if(path_has_changed) linphone_gtk_friend_list_update_button_display(friendlist);
+	return FALSE;
+}
+
+gboolean linphone_gtk_friend_list_leave_event_handler(GtkTreeView *friendlist, GdkEventCrossing *event) {
+	GtkTreePath *hovered_row = (GtkTreePath *)g_object_get_data(G_OBJECT(friendlist), "hovered_row");
+	if(hovered_row) {
+		g_object_set_data(G_OBJECT(friendlist), "hovered_row", NULL);
+		linphone_gtk_friend_list_update_button_display(friendlist);
+	}
+	return FALSE;
+}
+
+gboolean linphone_gtk_friend_list_motion_event_handler(GtkTreeView *friendlist, GdkEventMotion *event) {
+	gboolean path_has_changed = update_hovered_row_path(friendlist, event->x, event->y);
+	if(path_has_changed) linphone_gtk_friend_list_update_button_display(friendlist);
+	return FALSE;
+}
+
