@@ -29,6 +29,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 #include "status_notifier.h"
+#include <mediastreamer2/mscommon.h>
 
 typedef struct __LinphoneStatusIconDesc _LinphoneStatusIconDesc;
 
@@ -116,7 +117,7 @@ static gboolean _linphone_status_icon_desc_is_supported(
 	gboolean *result,
 	LinphoneStatusIconDescIsSupportedResultCb cb,
 	void *user_data) {
-	
+
 	return desc->is_supported(desc, result, cb, user_data);
 }
 
@@ -130,7 +131,7 @@ static void _linphone_status_icon_desc_is_supported_result_cb(
 	const _LinphoneStatusIconDesc *desc,
 	gboolean result,
 	_LinphoneStatusIconDescSearchCtx *ctx) {
-	
+
 	if(!result) {
 		ctx->i = g_slist_next(ctx->i);
 		for(; ctx->i; ctx->i = g_slist_next(ctx->i)) {
@@ -139,12 +140,20 @@ static void _linphone_status_icon_desc_is_supported_result_cb(
 				&result,
 				(LinphoneStatusIconDescIsSupportedResultCb)_linphone_status_icon_desc_is_supported_result_cb,
 				ctx)) {
-			
+
 				if(result) break;
 			} else return;
 		}
 	}
-	if(ctx->cb) ctx->cb((const _LinphoneStatusIconDesc *)g_slist_nth_data(ctx->i, 0), ctx->user_data);
+
+	if(ctx->i) {
+		const _LinphoneStatusIconDesc *desc = (const _LinphoneStatusIconDesc *)g_slist_nth_data(ctx->i, 0);
+		ms_message("StatusIcon: found implementation: %s", desc->impl_name);
+		if(ctx->cb) ctx->cb(desc, ctx->user_data);
+	} else {
+		g_warning("StatusIcon: no implementation found");
+	}
+
 	g_free(ctx);
 }
 
@@ -152,29 +161,33 @@ static gboolean _linphone_status_icon_find_first_available_impl(
 	const _LinphoneStatusIconDesc **desc,
 	LinphoneStatusIconDescFindResultCb cb,
 	void *user_data) {
-	
+
 	gboolean result;
 	_LinphoneStatusIconDescSearchCtx *ctx = g_new0(_LinphoneStatusIconDescSearchCtx, 1);
 	ctx->cb = cb;
 	ctx->user_data = user_data;
-	
+
+	ms_message("StatusIcon: looking for implementation...");
+
 	for(ctx->i=_linphone_status_icon_impls; ctx->i; ctx->i = g_slist_next(ctx->i)) {
 		if(_linphone_status_icon_desc_is_supported(
 			(const _LinphoneStatusIconDesc *)g_slist_nth_data(ctx->i, 0),
 			&result,
 			(LinphoneStatusIconDescIsSupportedResultCb)_linphone_status_icon_desc_is_supported_result_cb,
 			ctx)) {
-			
+
 			if(result) {
 				*desc = (const _LinphoneStatusIconDesc *)g_slist_nth_data(ctx->i, 0);
+				ms_message("StatusIcon: found implementation: %s", (*desc)->impl_name);
 				goto sync_return;
 			}
 		} else {
 			return 0;
 		}
 	}
+	g_warning("StatusIcon: no implementation found");
 	*desc = NULL;
-	
+
 sync_return:
 	g_free(ctx);
 	return 1;
@@ -195,7 +208,7 @@ static LinphoneStatusIcon *_linphone_status_icon_new(const _LinphoneStatusIconDe
 }
 
 static void _linphone_status_icon_free(LinphoneStatusIcon *obj) {
-	if(obj->desc->uninit) obj->desc->uninit(obj->data);
+	if(obj->desc->uninit) obj->desc->uninit(obj);
 	if(obj->params) linphone_status_icon_params_unref(obj->params);
 	g_free(obj);
 }
@@ -205,11 +218,13 @@ const char *linphone_status_icon_get_implementation_name(const LinphoneStatusIco
 }
 
 void linphone_status_icon_start(LinphoneStatusIcon *obj, LinphoneStatusIconParams *params) {
+	ms_message("StatusIcon: starting status icon");
 	obj->params = linphone_status_icon_params_ref(params);
 	if(obj->desc->start) obj->desc->start(obj);
 }
 
 void linphone_status_icon_enable_blinking(LinphoneStatusIcon *obj, gboolean enable) {
+	ms_message("StatusIcon: blinking set to %s", enable ? "TRUE" : "FALSE");
 	if(obj->desc->enable_blinking) obj->desc->enable_blinking(obj, enable);
 }
 
@@ -251,12 +266,15 @@ void _linphone_status_icon_create_implementations_list(void) {
 gboolean linphone_status_icon_init(LinphoneStatusIconReadyCb ready_cb, void *user_data) {
 	const _LinphoneStatusIconDesc *desc;
 	void **ctx;
-	
+
+	ms_message("StatusIcon: Initialising");
+
 	_linphone_status_icon_create_implementations_list();
-	
+
 	ctx = g_new(void *, 2);
 	ctx[0] = ready_cb;
 	ctx[1] = user_data;
+
 	if(_linphone_status_icon_find_first_available_impl(&desc, _linphone_status_icon_init_cb, ctx)) {
 		_linphone_status_icon_selected_desc = desc;
 		g_free(ctx);
@@ -279,6 +297,7 @@ void linphone_status_icon_uninit(void) {
 LinphoneStatusIcon *linphone_status_icon_get(void) {
 	if(_linphone_status_icon_instance == NULL) {
 		if(_linphone_status_icon_selected_desc)
+			ms_message("StatusIcon: instanciating singleton");
 			_linphone_status_icon_instance = _linphone_status_icon_new(_linphone_status_icon_selected_desc);
 	}
 	return _linphone_status_icon_instance;
@@ -297,22 +316,20 @@ static void _linphone_status_icon_impl_gtk_popup_menu(GtkStatusIcon *status_icon
 }
 
 static void _linphone_status_icon_impl_gtk_init(LinphoneStatusIcon *si) {
-	const char *icon_path=linphone_gtk_get_ui_config("icon",LINPHONE_ICON);
-	const char *call_icon_path=linphone_gtk_get_ui_config("start_call_icon","startcall-green.png");
-	GdkPixbuf *pbuf=create_pixbuf(icon_path);
-	GtkStatusIcon *icon=gtk_status_icon_new_from_pixbuf(pbuf);
+	const char *icon_name=linphone_gtk_get_ui_config("icon_name",LINPHONE_ICON_NAME);
+	const char *blinking_icon_name=linphone_gtk_get_ui_config("binking_status_icon_name","linphone-start-call");
+	GtkStatusIcon *icon=gtk_status_icon_new_from_icon_name(icon_name);
 	g_signal_connect_swapped(G_OBJECT(icon),"activate", G_CALLBACK(_linphone_status_icon_impl_gtk_on_click_cb), si);
 	g_signal_connect(G_OBJECT(icon), "popup-menu", G_CALLBACK(_linphone_status_icon_impl_gtk_popup_menu), si);
-	g_object_set_data_full(G_OBJECT(icon),"icon",pbuf, g_object_unref);
-	pbuf=create_pixbuf(call_icon_path);
-	g_object_set_data_full(G_OBJECT(icon),"call_icon",pbuf, g_object_unref);
+	g_object_set_data_full(G_OBJECT(icon), "icon", g_strdup(icon_name), g_free);
+	g_object_set_data_full(G_OBJECT(icon), "call_icon", g_strdup(blinking_icon_name), g_free);
 	si->data = icon;
 }
 
-// static void _linphone_status_icon_impl_gtk_uninit(LinphoneStatusIcon *si) {
-// 	GtkStatusIcon *icon = GTK_STATUS_ICON(si->data);
-// 	gtk_status_icon_set_visible(icon, FALSE);
-// }
+static void _linphone_status_icon_impl_gtk_uninit(LinphoneStatusIcon *si) {
+	GtkStatusIcon *icon = GTK_STATUS_ICON(si->data);
+	gtk_status_icon_set_visible(icon, FALSE);
+}
 
 static void _linphone_status_icon_impl_gtk_start(LinphoneStatusIcon *si) {
 	GtkStatusIcon *icon = GTK_STATUS_ICON(si->data);
@@ -325,13 +342,13 @@ static void _linphone_status_icon_impl_gtk_start(LinphoneStatusIcon *si) {
 }
 
 static gboolean _linphone_status_icon_impl_gtk_do_icon_blink_cb(GtkStatusIcon *gi){
-	GdkPixbuf *call_icon=g_object_get_data(G_OBJECT(gi),"call_icon");
-	GdkPixbuf *normal_icon=g_object_get_data(G_OBJECT(gi),"icon");
-	GdkPixbuf *cur_icon=gtk_status_icon_get_pixbuf(gi);
-	if (cur_icon==call_icon){
-		gtk_status_icon_set_from_pixbuf(gi,normal_icon);
+	const gchar *call_icon = (const gchar *)g_object_get_data(G_OBJECT(gi),"call_icon");
+	const gchar *normal_icon = (const gchar *)g_object_get_data(G_OBJECT(gi),"icon");
+	const gchar *cur_icon = (const gchar *)gtk_status_icon_get_icon_name(gi);
+	if (cur_icon == call_icon){
+		gtk_status_icon_set_from_icon_name(gi,normal_icon);
 	}else{
-		gtk_status_icon_set_from_pixbuf(gi,call_icon);
+		gtk_status_icon_set_from_icon_name(gi,call_icon);
 	}
 	return TRUE;
 }
@@ -344,10 +361,10 @@ static void _linphone_status_icon_impl_enable_blinking(LinphoneStatusIcon *si, g
 		tout=g_timeout_add(500,(GSourceFunc)_linphone_status_icon_impl_gtk_do_icon_blink_cb,icon);
 		g_object_set_data(G_OBJECT(icon),"timeout",GINT_TO_POINTER(tout));
 	}else if (!val && tout!=0){
-		GdkPixbuf *normal_icon=g_object_get_data(G_OBJECT(icon),"icon");
+		const gchar *normal_icon = (const gchar *)g_object_get_data(G_OBJECT(icon),"icon");
 		g_source_remove(tout);
 		g_object_set_data(G_OBJECT(icon),"timeout",NULL);
-		gtk_status_icon_set_from_pixbuf(icon,normal_icon);
+		gtk_status_icon_set_from_icon_name(icon,normal_icon);
 	}
 }
 
@@ -356,7 +373,7 @@ static gboolean _linphone_status_icon_impl_is_supported(
 	gboolean *result,
 	LinphoneStatusIconDescIsSupportedResultCb cb,
 	void *user_data) {
-	
+
 	*result = 1;
 	return 1;
 }
@@ -364,7 +381,7 @@ static gboolean _linphone_status_icon_impl_is_supported(
 static const _LinphoneStatusIconDesc _linphone_status_icon_impl_gtk_desc = {
 	"gtk_status_icon",
 	_linphone_status_icon_impl_gtk_init,
-	NULL,
+	_linphone_status_icon_impl_gtk_uninit,
 	_linphone_status_icon_impl_gtk_start,
 	_linphone_status_icon_impl_enable_blinking,
 	_linphone_status_icon_impl_is_supported
@@ -390,7 +407,7 @@ static gboolean _linphone_status_icon_impl_gtkosx_app_is_supported(
 	gboolean *result,
 	LinphoneStatusIconDescIsSupportedResultCb cb,
 	void *user_data) {
-	
+
 	*result = 1;
 	return 1;
 }
@@ -453,10 +470,10 @@ static void _linphone_status_icon_impl_sn_start(LinphoneStatusIcon *si) {
 	BcStatusNotifierParams *params;
 	BcStatusNotifierToolTip *tooltip = bc_status_notifier_tool_tip_new("linphone", si->params->title, si->params->desc);
 	BcStatusNotifierSignalsVTable vtable = {NULL};
-	
+
 	vtable.activate_called_cb = _linphone_status_icon_impl_sn_activated_cb;
 	vtable.context_menu_called_cb = _linphone_status_icon_impl_sn_menu_called_cb;
-	
+
 	params = bc_status_notifier_params_new();
 	bc_status_notifier_params_set_dbus_prefix(params, "org.kde");
 	bc_status_notifier_params_set_category(params, BcStatusNotifierCategoryCommunications);
@@ -465,15 +482,15 @@ static void _linphone_status_icon_impl_sn_start(LinphoneStatusIcon *si) {
 	bc_status_notifier_params_set_icon_name(params, "linphone");
 	bc_status_notifier_params_set_tool_tip(params, tooltip);
 	bc_status_notifier_params_set_vtable(params, &vtable, si);
-	
+
 	bc_status_notifier_start(sn, params, NULL, NULL);
-	
+
 	bc_status_notifier_tool_tip_unref(tooltip);
 	bc_status_notifier_params_unref(params);
 }
 
 static void _linphone_status_icon_impl_sn_enable_blinking(LinphoneStatusIcon *si, gboolean val) {
-	BcStatusNotifier *sn = (BcStatusNotifier *)si->data; 
+	BcStatusNotifier *sn = (BcStatusNotifier *)si->data;
 	if(val) {
 		bc_status_notifier_update_status(sn, BcStatusNotifierStatusNeedsAttention);
 	} else {
@@ -494,16 +511,16 @@ static gboolean _linphone_status_icon_impl_sn_is_supported(
 	gboolean *result,
 	LinphoneStatusIconDescIsSupportedResultCb cb,
 	void *user_data) {
-	
+
 	_LinphoneStatusIconDesc *desc2;
 	void **data;
 	const char *desktop = g_getenv("XDG_CURRENT_DESKTOP");
-	
+
 	if(desktop == NULL || g_strcmp0(desktop, "KDE") != 0) {
 		*result = FALSE;
 		return TRUE;
 	}
-	
+
 	desc2 = g_new(_LinphoneStatusIconDesc, 1);
 	*desc2 = *desc;
 	data = g_new(void *, 3);
