@@ -59,6 +59,10 @@ struct SalCustomHeader;
 
 typedef struct SalCustomHeader SalCustomHeader;
 
+struct SalCustomSdpAttribute;
+
+typedef struct SalCustomSdpAttribute SalCustomSdpAttribute;
+
 struct addrinfo;
 
 typedef enum {
@@ -68,15 +72,17 @@ typedef enum {
 	SalTransportDTLS, /*DTLS*/
 }SalTransport;
 
-#define SAL_MEDIA_DESCRIPTION_UNCHANGED			0x00
-#define SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED		(1)
-#define SAL_MEDIA_DESCRIPTION_CODEC_CHANGED		(1<<1)
-#define SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED	(1<<2)
-#define SAL_MEDIA_DESCRIPTION_CRYPTO_POLICY_CHANGED	(1<<3)
-#define SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED		(1<<4)
+#define SAL_MEDIA_DESCRIPTION_UNCHANGED						0x00
+#define SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED				(1)
+#define SAL_MEDIA_DESCRIPTION_CODEC_CHANGED					(1<<1)
+#define SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED			(1<<2)
+#define SAL_MEDIA_DESCRIPTION_CRYPTO_POLICY_CHANGED			(1<<3)
+#define SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED				(1<<4)
+#define SAL_MEDIA_DESCRIPTION_NETWORK_XXXCAST_CHANGED		(1<<5) /* use to notify when switching from multicast to unicast*/
+#define SAL_MEDIA_DESCRIPTION_FORCE_STREAM_RECONSTRUCTION	(1<<6) /* use force graph reconstruction*/
+#define SAL_MEDIA_DESCRIPTION_ICE_RESTART_DETECTED			(1<<7)
+
 
-#define SAL_MEDIA_DESCRIPTION_CHANGED		(SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED | SAL_MEDIA_DESCRIPTION_CODEC_CHANGED |\
-						SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED |SAL_MEDIA_DESCRIPTION_CRYPTO_POLICY_CHANGED | SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED)
 
 const char* sal_transport_to_string(SalTransport transport);
 SalTransport sal_transport_parse(const char*);
@@ -116,8 +122,8 @@ void sal_address_set_password(SalAddress *addr, const char *passwd);
 const char *sal_address_get_password(const SalAddress *addr);
 void sal_address_set_header(SalAddress *addr, const char *header_name, const char *header_value);
 
-Sal * sal_init();
-void sal_uninit(Sal* sal);
+LINPHONE_PUBLIC Sal * sal_init();
+LINPHONE_PUBLIC void sal_uninit(Sal* sal);
 void sal_set_user_pointer(Sal *sal, void *user_data);
 void *sal_get_user_pointer(const Sal *sal);
 
@@ -125,6 +131,7 @@ void *sal_get_user_pointer(const Sal *sal);
 typedef enum {
 	SalAudio,
 	SalVideo,
+	SalText,
 	SalOther
 } SalStreamType;
 const char* sal_stream_type_to_string(SalStreamType type);
@@ -218,7 +225,7 @@ typedef struct SalStreamDescription{
 	char rtp_addr[64];
 	char rtcp_addr[64];
 	unsigned int rtp_ssrc;
-	char rtcp_cname[255];
+	char rtcp_cname[256];
 	int rtp_port;
 	int rtcp_port;
 	MSList *payloads; /*<list of PayloadType */
@@ -229,7 +236,9 @@ typedef struct SalStreamDescription{
 	SalSrtpCryptoAlgo crypto[SAL_CRYPTO_ALGO_MAX];
 	unsigned int crypto_local_tag;
 	int max_rate;
+	OrtpRtcpFbConfiguration rtcp_fb;
 	OrtpRtcpXrConfiguration rtcp_xr;
+	SalCustomSdpAttribute *custom_sdp_attributes;
 	SalIceCandidate ice_candidates[SAL_MEDIA_DESCRIPTION_MAX_ICE_CANDIDATES];
 	SalIceRemoteCandidate ice_remote_candidates[SAL_MEDIA_DESCRIPTION_MAX_ICE_REMOTE_CANDIDATES];
 	char ice_ufrag[SAL_MEDIA_DESCRIPTION_MAX_ICE_UFRAG_LEN];
@@ -259,6 +268,7 @@ typedef struct SalMediaDescription{
 	unsigned int session_id;
 	SalStreamDir dir;
 	SalStreamDescription streams[SAL_MEDIA_DESCRIPTION_MAX_STREAMS];
+	SalCustomSdpAttribute *custom_sdp_attributes;
 	OrtpRtcpXrConfiguration rtcp_xr;
 	char ice_ufrag[SAL_MEDIA_DESCRIPTION_MAX_ICE_UFRAG_LEN];
 	char ice_pwd[SAL_MEDIA_DESCRIPTION_MAX_ICE_PWD_LEN];
@@ -288,9 +298,9 @@ SalMediaDescription * sal_media_description_ref(SalMediaDescription *md);
 void sal_media_description_unref(SalMediaDescription *md);
 bool_t sal_media_description_empty(const SalMediaDescription *md);
 int sal_media_description_equals(const SalMediaDescription *md1, const SalMediaDescription *md2);
+char * sal_media_description_print_differences(int result);
 bool_t sal_media_description_has_dir(const SalMediaDescription *md, SalStreamDir dir);
-SalStreamDescription *sal_media_description_find_stream(SalMediaDescription *md,
-    SalMediaProto proto, SalStreamType type);
+LINPHONE_PUBLIC SalStreamDescription *sal_media_description_find_stream(SalMediaDescription *md, SalMediaProto proto, SalStreamType type);
 unsigned int sal_media_description_nb_active_streams_of_type(SalMediaDescription *md, SalStreamType type);
 SalStreamDescription * sal_media_description_get_active_stream_of_type(SalMediaDescription *md, SalStreamType type, unsigned int idx);
 SalStreamDescription * sal_media_description_find_secure_stream_of_type(SalMediaDescription *md, SalStreamType type);
@@ -355,7 +365,8 @@ typedef enum SalReason{
 	SalReasonNotImplemented,
 	SalReasonBadGateway,
 	SalReasonServerTimeout,
-	SalReasonIOError
+	SalReasonIOError,
+	SalReasonInternalError
 }SalReason;
 
 const char* sal_reason_to_string(const SalReason reason);
@@ -567,7 +578,7 @@ void sal_set_callbacks(Sal *ctx, const SalCallbacks *cbs);
 int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int is_tunneled);
 int sal_get_listening_port(Sal *ctx, SalTransport tr);
 int sal_unlisten_ports(Sal *ctx);
-int sal_transport_available(Sal *ctx, SalTransport t);
+LINPHONE_PUBLIC int sal_transport_available(Sal *ctx, SalTransport t);
 void sal_set_dscp(Sal *ctx, int dscp);
 void sal_set_supported_tags(Sal *ctx, const char* tags);
 void sal_add_supported_tag(Sal *ctx, const char* tag);
@@ -649,6 +660,7 @@ const char *sal_op_get_remote_ua(const SalOp *op);
 void *sal_op_get_user_pointer(const SalOp *op);
 const char* sal_op_get_call_id(const SalOp *op);
 char* sal_op_get_dialog_id(const SalOp *op);
+bool_t sal_op_is_forked_of(const SalOp *op1, const SalOp *op2);
 
 const SalAddress* sal_op_get_service_route(const SalOp *op);
 void sal_op_set_service_route(SalOp *op,const SalAddress* service_route);
@@ -703,7 +715,7 @@ LINPHONE_PUBLIC void sal_default_set_sdp_handling(Sal* h, SalOpSDPHandling handl
 LINPHONE_PUBLIC void sal_call_set_sdp_handling(SalOp *h, SalOpSDPHandling handling) ;
 
 /*Registration*/
-int sal_register(SalOp *op, const char *proxy, const char *from, int expires);
+int sal_register(SalOp *op, const char *proxy, const char *from, int expires,SalAddress* old_contact);
 /*refresh a register, -1 mean use the last known value*/
 int sal_register_refresh(SalOp *op, int expires);
 int sal_unregister(SalOp *h);
@@ -755,8 +767,8 @@ SalPrivacy sal_op_get_privacy(const SalOp* op);
 
 
 
-#define payload_type_set_number(pt,n)		(pt)->user_data=(void*)((long)n);
-#define payload_type_get_number(pt)		((int)(long)(pt)->user_data)
+#define payload_type_set_number(pt,n)		(pt)->user_data=(void*)((intptr_t)n);
+#define payload_type_get_number(pt)		((int)(intptr_t)(pt)->user_data)
 
 /*misc*/
 void sal_get_default_local_ip(Sal *sal, int address_family, char *ip, size_t iplen);
@@ -777,8 +789,17 @@ const SalCustomHeader *sal_op_get_recv_custom_header(SalOp *op);
 
 void sal_op_set_sent_custom_header(SalOp *op, SalCustomHeader* ch);
 
-void sal_enable_logs();
-void sal_disable_logs();
+
+SalCustomSdpAttribute * sal_custom_sdp_attribute_append(SalCustomSdpAttribute *csa, const char *name, const char *value);
+const char * sal_custom_sdp_attribute_find(const SalCustomSdpAttribute *csa, const char *name);
+void sal_custom_sdp_attribute_free(SalCustomSdpAttribute *csa);
+SalCustomSdpAttribute * sal_custom_sdp_attribute_clone(const SalCustomSdpAttribute *csa);
+
+/** deprecated. use sal_set_log_level instead **/
+void sal_enable_log();
+/** deprecated. use sal_set_log_level instead **/
+void sal_disable_log();
+void sal_set_log_level(OrtpLogLevel level);
 
 /*internal API */
 void __sal_op_init(SalOp *b, Sal *sal);
@@ -796,6 +817,11 @@ LINPHONE_PUBLIC	void sal_set_recv_error(Sal *sal,int value);
 /*always answer 480 if value=true*/
 LINPHONE_PUBLIC	void sal_enable_unconditional_answer(Sal *sal,int value);
 
+LINPHONE_PUBLIC bool_t sal_pending_trans_checking_enabled(const Sal *sal) ;
+LINPHONE_PUBLIC int sal_enable_pending_trans_checking(Sal *sal, bool_t value) ;
+
+
+
 /*refresher retry after value in ms*/
 LINPHONE_PUBLIC	void sal_set_refresher_retry_after(Sal *sal,int value);
 LINPHONE_PUBLIC	int sal_get_refresher_retry_after(const Sal *sal);
@@ -831,6 +857,12 @@ void sal_end_background_task(unsigned long id);
 void sal_op_cnx_ip_to_0000_if_sendonly_enable(SalOp *sal,bool_t yesno);
 bool_t sal_op_cnx_ip_to_0000_if_sendonly_enabled(SalOp *sal);
 
+void sal_set_http_proxy_host(Sal *sal, const char *host) ;
+void sal_set_http_proxy_port(Sal *sal, int port) ;
+const char *sal_get_http_proxy_host(const Sal *sal);
+int sal_get_http_proxy_port(const Sal *sal);
+
+
 #endif
 
 
