@@ -16,8 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <stdio.h>
-#include "CUnit/Basic.h"
+
 #include "linphonecore.h"
 #include "private.h"
 #include "liblinphone_tester.h"
@@ -29,12 +28,14 @@ static LinphoneCoreManager* presence_linphone_core_manager_new(char* username) {
 	linphone_address_set_username(mgr->identity,username);
 	identity_char=linphone_address_as_string(mgr->identity);
 	linphone_core_set_primary_contact(mgr->lc,identity_char);
+	ms_free(identity_char);
 	return mgr;
 }
+
 void new_subscription_requested(LinphoneCore *lc, LinphoneFriend *lf, const char *url){
 	char* from=linphone_address_as_string(linphone_friend_get_address(lf));
 	stats* counters;
-	ms_message("New subscription request  from [%s]  url [%s]",from,url);
+	ms_message("New subscription request from [%s] url [%s]",from,url);
 	ms_free(from);
 	counters = get_stats(lc);
 	counters->number_of_NewSubscriptionRequest++;
@@ -142,11 +143,11 @@ static void simple_publish_with_expire(int expires) {
 	linphone_core_manager_destroy(marie);
 }
 
-static void simple_publish() {
+static void simple_publish(void) {
 	simple_publish_with_expire(-1);
 }
 
-static void publish_with_expires() {
+static void publish_with_expires(void) {
 	simple_publish_with_expire(1);
 }
 
@@ -163,17 +164,18 @@ static bool_t subscribe_to_callee_presence(LinphoneCoreManager* caller_mgr,Linph
 	linphone_friend_done(friend);
 
 	linphone_core_add_friend(caller_mgr->lc,friend);
+	linphone_friend_unref(friend);
 
 	result=wait_for(caller_mgr->lc,callee_mgr->lc,&caller_mgr->stat.number_of_LinphonePresenceActivityOnline,initial_caller.number_of_LinphonePresenceActivityOnline+1);
 	/*without proxy, callee cannot subscribe to caller
 	result&=wait_for(caller_mgr->lc,callee_mgr->lc,&callee_mgr->stat.number_of_LinphonePresenceActivityOnline,initial_callee.number_of_LinphonePresenceActivityOnline+1);
 	*/
 
-	CU_ASSERT_EQUAL(callee_mgr->stat.number_of_NewSubscriptionRequest,initial_callee.number_of_NewSubscriptionRequest+1);
+	BC_ASSERT_EQUAL(callee_mgr->stat.number_of_NewSubscriptionRequest,initial_callee.number_of_NewSubscriptionRequest+1, int, "%d");
 	/*without proxy, callee cannot subscribe to caller
-	CU_ASSERT_EQUAL(callee_mgr->stat.number_of_NotifyReceived,initial_callee.number_of_NotifyReceived+1);
+	BC_ASSERT_EQUAL(callee_mgr->stat.number_of_NotifyReceived,initial_callee.number_of_NotifyReceived+1, int, "%d");
 	*/
-	CU_ASSERT_EQUAL(caller_mgr->stat.number_of_NotifyReceived,initial_caller.number_of_NotifyReceived+1);
+	BC_ASSERT_EQUAL(caller_mgr->stat.number_of_NotifyReceived,initial_caller.number_of_NotifyReceived+1, int, "%d");
 
 	ms_free(identity);
 	return result;
@@ -181,33 +183,34 @@ static bool_t subscribe_to_callee_presence(LinphoneCoreManager* caller_mgr,Linph
 }
 static void subscribe_failure_handle_by_app(void) {
 	LinphoneCoreManager* marie = linphone_core_manager_new("marie_rc");
-	LinphoneCoreManager* pauline = linphone_core_manager_new("pauline_rc");
+	LinphoneCoreManager* pauline = linphone_core_manager_new("pauline_tcp_rc");
 	LinphoneProxyConfig* config;
 	LinphoneFriend* lf;
 	char* lf_identity=linphone_address_as_string_uri_only(pauline->identity);
-	 linphone_core_get_default_proxy(marie->lc,&config);
+	
+	linphone_core_get_default_proxy(marie->lc,&config);
 
-	CU_ASSERT_TRUE(subscribe_to_callee_presence(marie,pauline));
+	BC_ASSERT_TRUE(subscribe_to_callee_presence(marie,pauline));
 	wait_for(marie->lc,pauline->lc,&pauline->stat.number_of_NewSubscriptionRequest,1); /*just to wait for unsubscription even if not notified*/
 
 	sal_set_recv_error(marie->lc->sal, 0); /*simulate an error*/
 
-	CU_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphoneRegistrationProgress,2));
-	CU_ASSERT_EQUAL(linphone_proxy_config_get_error(config),LinphoneReasonIOError);
+	BC_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphoneRegistrationProgress,2));
+	BC_ASSERT_EQUAL(linphone_proxy_config_get_error(config),LinphoneReasonIOError, int, "%d");
 	sal_set_recv_error(marie->lc->sal, 1);
 
 	lf = linphone_core_get_friend_by_address(marie->lc,lf_identity);
 	linphone_friend_edit(lf);
 	linphone_friend_enable_subscribes(lf,FALSE); /*disable subscription*/
 	linphone_friend_done(lf);
-	CU_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphoneRegistrationOk,2)); /*wait for register ok*/
+	BC_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphoneRegistrationOk,2)); /*wait for register ok*/
 	linphone_friend_edit(lf);
 	linphone_friend_enable_subscribes(lf,TRUE);
 	linphone_friend_done(lf);
-	CU_ASSERT_FALSE(wait_for(marie->lc,pauline->lc,&pauline->stat.number_of_NewSubscriptionRequest,2)); /*just to wait for unsubscription even if not notified*/
+	BC_ASSERT_FALSE(wait_for(marie->lc,pauline->lc,&pauline->stat.number_of_NewSubscriptionRequest,2)); /*just to wait for unsubscription even if not notified*/
 
 	linphone_core_manager_destroy(marie);
-	CU_ASSERT_FALSE(wait_for(NULL,pauline->lc,&pauline->stat.number_of_NewSubscriptionRequest,3)); /*just to wait for unsubscription even if not notified*/
+	BC_ASSERT_FALSE(wait_for(NULL,pauline->lc,&pauline->stat.number_of_NewSubscriptionRequest,3)); /*just to wait for unsubscription even if not notified*/
 
 	linphone_core_manager_destroy(pauline);
 }
@@ -216,12 +219,12 @@ static void simple_subscribe(void) {
 	LinphoneCoreManager* marie = presence_linphone_core_manager_new("marie");
 	LinphoneCoreManager* pauline = presence_linphone_core_manager_new("pauline");
 
-	CU_ASSERT_TRUE(subscribe_to_callee_presence(marie,pauline));
+	BC_ASSERT_TRUE(subscribe_to_callee_presence(marie,pauline));
 
 
 	linphone_core_manager_destroy(marie);
 	/*unsubscribe is not reported ?*/
-	CU_ASSERT_FALSE(wait_for(NULL,pauline->lc,&pauline->stat.number_of_NewSubscriptionRequest,2)); /*just to wait for unsubscription even if not notified*/
+	BC_ASSERT_FALSE(wait_for(NULL,pauline->lc,&pauline->stat.number_of_NewSubscriptionRequest,2)); /*just to wait for unsubscription even if not notified*/
 
 	linphone_core_manager_destroy(pauline);
 }
@@ -233,6 +236,7 @@ static void unsubscribe_while_subscribing(void) {
 	linphone_friend_enable_subscribes(friend,TRUE);
 	linphone_friend_done(friend);
 	linphone_core_add_friend(marie->lc,friend);
+	linphone_friend_unref(friend);
 	linphone_core_iterate(marie->lc);
 	linphone_core_manager_destroy(marie);
 }
@@ -244,18 +248,18 @@ static void call_with_presence(void) {
 	LinphoneCoreManager* pauline = presence_linphone_core_manager_new("pauline");
 	LinphoneVideoPolicy pol={0};
 	linphone_core_set_video_policy(marie->lc,&pol);
-	CU_ASSERT_TRUE(subscribe_to_callee_presence(marie,pauline));
-	CU_ASSERT_TRUE(subscribe_to_callee_presence(pauline,marie));
+	BC_ASSERT_TRUE(subscribe_to_callee_presence(marie,pauline));
+	BC_ASSERT_TRUE(subscribe_to_callee_presence(pauline,marie));
 
-	CU_ASSERT_TRUE(call(marie,pauline));
-	CU_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnThePhone,1));
-	CU_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&pauline->stat.number_of_LinphonePresenceActivityOnThePhone,1));
+	BC_ASSERT_TRUE(call(marie,pauline));
+	BC_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnThePhone,1));
+	BC_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&pauline->stat.number_of_LinphonePresenceActivityOnThePhone,1));
 
 	reset_counters(&marie->stat);
 	reset_counters(&pauline->stat);
 	linphone_core_terminate_all_calls(marie->lc);
-	CU_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&pauline->stat.number_of_LinphonePresenceActivityOnline,1));
-	CU_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnline,1));
+	BC_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&pauline->stat.number_of_LinphonePresenceActivityOnline,1));
+	BC_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnline,1));
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
 }
@@ -277,48 +281,48 @@ static void presence_information(void) {
 	char *contact2;
 	time_t current_timestamp, presence_timestamp;
 
-	CU_ASSERT_TRUE(subscribe_to_callee_presence(marie, pauline));
+	BC_ASSERT_TRUE(subscribe_to_callee_presence(marie, pauline));
 
 	/* Presence activity without description. */
 	presence = linphone_presence_model_new_with_activity(LinphonePresenceActivityDinner, NULL);
 	linphone_core_set_presence_model(pauline->lc, presence);
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityDinner,1);
-	CU_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityDinner, 1);
+	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityDinner, 1, int, "%d");
 	activity = linphone_presence_model_get_activity(marie->stat.last_received_presence);
-	CU_ASSERT_PTR_NOT_NULL(activity);
-	CU_ASSERT_EQUAL(linphone_presence_activity_get_type(activity), LinphonePresenceActivityDinner);
+	BC_ASSERT_PTR_NOT_NULL(activity);
+	BC_ASSERT_EQUAL(linphone_presence_activity_get_type(activity), LinphonePresenceActivityDinner, int, "%d");
 	description = linphone_presence_activity_get_description(activity);
-	CU_ASSERT_PTR_NULL(description);
+	BC_ASSERT_PTR_NULL(description);
 
 	/* Presence activity with description. */
 	presence = linphone_presence_model_new_with_activity(LinphonePresenceActivitySteering, bike_description);
 	linphone_core_set_presence_model(pauline->lc, presence);
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivitySteering,1);
-	CU_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivitySteering, 1);
+	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivitySteering, 1, int, "%d");
 	activity = linphone_presence_model_get_activity(marie->stat.last_received_presence);
-	CU_ASSERT_PTR_NOT_NULL(activity);
-	CU_ASSERT_EQUAL(linphone_presence_activity_get_type(activity), LinphonePresenceActivitySteering);
+	BC_ASSERT_PTR_NOT_NULL(activity);
+	BC_ASSERT_EQUAL(linphone_presence_activity_get_type(activity), LinphonePresenceActivitySteering, int, "%d");
 	description = linphone_presence_activity_get_description(activity);
-	CU_ASSERT_PTR_NOT_NULL(description);
-	if (description != NULL) CU_ASSERT_EQUAL(strcmp(description, bike_description), 0);
+	BC_ASSERT_PTR_NOT_NULL(description);
+	if (description != NULL) BC_ASSERT_STRING_EQUAL(description, bike_description);
 
 	/* Presence activity with description and note. */
 	presence = linphone_presence_model_new_with_activity_and_note(LinphonePresenceActivityVacation, NULL, vacation_note, vacation_lang);
 	linphone_core_set_presence_model(pauline->lc, presence);
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityVacation,1);
-	CU_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityVacation, 1);
+	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityVacation, 1, int, "%d");
 	activity = linphone_presence_model_get_activity(marie->stat.last_received_presence);
-	CU_ASSERT_PTR_NOT_NULL(activity);
-	CU_ASSERT_EQUAL(linphone_presence_activity_get_type(activity), LinphonePresenceActivityVacation);
+	BC_ASSERT_PTR_NOT_NULL(activity);
+	BC_ASSERT_EQUAL(linphone_presence_activity_get_type(activity), LinphonePresenceActivityVacation, int, "%d");
 	description = linphone_presence_activity_get_description(activity);
-	CU_ASSERT_PTR_NULL(description);
+	BC_ASSERT_PTR_NULL(description);
 	note = linphone_presence_model_get_note(marie->stat.last_received_presence, NULL);
-	CU_ASSERT_PTR_NOT_NULL(note);
+	BC_ASSERT_PTR_NOT_NULL(note);
 	if (note != NULL) {
 		note_content = linphone_presence_note_get_content(note);
-		CU_ASSERT_PTR_NOT_NULL(note_content);
+		BC_ASSERT_PTR_NOT_NULL(note_content);
 		if (note_content != NULL) {
-			CU_ASSERT_EQUAL(strcmp(note_content, vacation_note), 0);
+			BC_ASSERT_STRING_EQUAL(note_content, vacation_note);
 		}
 	}
 
@@ -327,11 +331,11 @@ static void presence_information(void) {
 	linphone_presence_model_set_contact(presence, contact);
 	linphone_core_set_presence_model(pauline->lc, presence);
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnThePhone,1);
-	CU_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityOnThePhone, 1);
+	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityOnThePhone, 1, int, "%d");
 	contact2 = linphone_presence_model_get_contact(presence);
-	CU_ASSERT_PTR_NOT_NULL(contact2);
+	BC_ASSERT_PTR_NOT_NULL(contact2);
 	if (contact2 != NULL) {
-		CU_ASSERT_EQUAL(strcmp(contact, contact2), 0);
+		BC_ASSERT_STRING_EQUAL(contact, contact2);
 		ms_free(contact2);
 	}
 
@@ -340,13 +344,91 @@ static void presence_information(void) {
 	presence = linphone_presence_model_new_with_activity(LinphonePresenceActivityShopping, NULL);
 	linphone_core_set_presence_model(pauline->lc, presence);
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityShopping,1);
-	CU_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityShopping, 1);
+	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityShopping, 1, int, "%d");
 	presence_timestamp = linphone_presence_model_get_timestamp(presence);
-	CU_ASSERT_TRUE(presence_timestamp >= current_timestamp);
+	BC_ASSERT_GREATER((unsigned)presence_timestamp , (unsigned)current_timestamp, unsigned, "%u");
 
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
 }
+
+
+static void subscribe_presence_forked(){
+	LinphoneCoreManager* marie = linphone_core_manager_new("marie_rc");
+	LinphoneCoreManager* pauline1 = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	LinphoneCoreManager* pauline2 = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	LinphoneFriend *lf;
+	MSList *lcs = NULL;
+	
+	lcs = ms_list_append(lcs, marie->lc);
+	lcs = ms_list_append(lcs, pauline1->lc);
+	lcs = ms_list_append(lcs, pauline2->lc);
+	
+	lf = linphone_core_create_friend(marie->lc);
+	linphone_friend_set_address(lf, pauline1->identity);
+	linphone_friend_enable_subscribes(lf, TRUE);
+	
+	linphone_core_add_friend(marie->lc, lf);
+	linphone_friend_unref(lf);
+	
+	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_NewSubscriptionRequest,1, 10000));
+	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline2->stat.number_of_NewSubscriptionRequest,1, 2000));
+	/*we should get two notifies*/
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphonePresenceActivityOnline,2, 10000));
+	
+	/*marie also shall receive two SUBSCRIBEs from the two paulines, but won't be notified to the app since 
+	 Marie set Pauline as a friend.*/
+	BC_ASSERT_EQUAL(marie->stat.number_of_NewSubscriptionRequest, 0, int, "%d");
+	/*and the two paulines shall be notified of marie's presence*/
+	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_LinphonePresenceActivityOnline,1, 3000));
+	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline2->stat.number_of_LinphonePresenceActivityOnline,1, 2000));
+	
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline1);
+	linphone_core_manager_destroy(pauline2);
+	
+	ms_list_free(lcs);
+}
+
+static void subscribe_presence_expired(){
+	LinphoneCoreManager* marie = linphone_core_manager_new("marie_rc");
+	LinphoneCoreManager* pauline1 = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	LinphoneFriend *lf;
+	MSList *lcs = NULL;
+	
+	lcs = ms_list_append(lcs, marie->lc);
+	lcs = ms_list_append(lcs, pauline1->lc);
+	
+	lp_config_set_int(marie->lc->config, "sip", "subscribe_expires", 10);
+	
+	lf = linphone_core_create_friend(marie->lc);
+	linphone_friend_set_address(lf, pauline1->identity);
+	linphone_friend_enable_subscribes(lf, TRUE);
+	
+	linphone_core_add_friend(marie->lc, lf);
+	linphone_friend_unref(lf);
+	
+	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_NewSubscriptionRequest,1, 5000));
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphonePresenceActivityOnline,1, 2000));
+	
+	lf = linphone_core_find_friend(pauline1->lc, marie->identity);
+	BC_ASSERT_PTR_NOT_NULL(lf->insubs);
+	/*marie comes offline suddenly*/
+	linphone_core_set_network_reachable(marie->lc, FALSE);
+	/*after a certain time, pauline shall see the incoming SUBSCRIBE expired*/
+	wait_for_list(lcs,NULL, 0, 11000);
+	
+	BC_ASSERT_PTR_NULL(lf->insubs);
+	/*just make network reachable so that marie can unregister properly*/
+	linphone_core_set_network_reachable(marie->lc, TRUE);
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneRegistrationOk,2, 10000));
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline1);
+	
+	ms_list_free(lcs);
+}
+
+
 #define USE_PRESENCE_SERVER 0
 
 #if USE_PRESENCE_SERVER
@@ -367,7 +449,7 @@ static void test_subscribe_notify_publish(void) {
 
 	/*wait for subscribe acknowledgment*/
 	wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,1,2000);
-	CU_ASSERT_EQUAL(LinphoneStatusOffline,linphone_friend_get_status(lf));
+	BC_ASSERT_EQUAL(LinphoneStatusOffline,linphone_friend_get_status(lf), int, "%d");
 
 	/*enable publish*/
 
@@ -380,23 +462,23 @@ static void test_subscribe_notify_publish(void) {
 
 	/*wait for marie status*/
 	wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,2,2000);
-	CU_ASSERT_EQUAL(LinphoneStatusOnline,linphone_friend_get_status(lf));
+	BC_ASSERT_EQUAL(LinphoneStatusOnline,linphone_friend_get_status(lf), int, "%d");
 
 	presence =linphone_presence_model_new_with_activity(LinphonePresenceActivityBusy,NULL);
 	linphone_core_set_presence_model(marie->lc,presence);
 
 	/*wait for new status*/
 	wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,3,2000);
-	CU_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf));
+	BC_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf), int, "%d");
 
 	/*wait for refresh*/
 	wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,4,5000);
-	CU_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf));
+	BC_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf), int, "%d");
 
 	/*linphone_core_remove_friend(pauline->lc,lf);*/
 	/*wait for final notify*/
 	/*wait_for_until(pauline->lc,marie->lc,&pauline->stat.number_of_NotifyReceived,4,5000);
-	CU_ASSERT_EQUAL(LinphonePresenceActivityOffline,linphone_friend_get_status(lf));
+	BC_ASSERT_EQUAL(LinphonePresenceActivityOffline,linphone_friend_get_status(lf), int, "%d");
 	 */
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
@@ -426,7 +508,7 @@ static void test_forked_subscribe_notify_publish(void) {
 
 	/*wait for subscribe acknowledgment*/
 	wait_for_list(lcs,&pauline->stat.number_of_NotifyReceived,1,2000);
-	CU_ASSERT_EQUAL(LinphoneStatusOffline,linphone_friend_get_status(lf));
+	BC_ASSERT_EQUAL(LinphoneStatusOffline,linphone_friend_get_status(lf), int, "%d");
 
 	/*enable publish*/
 
@@ -445,21 +527,21 @@ static void test_forked_subscribe_notify_publish(void) {
 
 	/*wait for marie status*/
 	wait_for_list(lcs,&pauline->stat.number_of_NotifyReceived,3,2000);
-	CU_ASSERT_EQUAL(LinphoneStatusOnline,linphone_friend_get_status(lf));
+	BC_ASSERT_EQUAL(LinphoneStatusOnline,linphone_friend_get_status(lf), int, "%d");
 
 	presence =linphone_presence_model_new_with_activity(LinphonePresenceActivityBusy,NULL);
 	linphone_core_set_presence_model(marie->lc,presence);
 
 	/*wait for new status*/
 	wait_for_list(lcs,&pauline->stat.number_of_NotifyReceived,4,2000);
-	CU_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf));
+	BC_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf), int, "%d");
 
 
 	presence =linphone_presence_model_new_with_activity(  LinphonePresenceActivityMeeting,NULL);
 	linphone_core_set_presence_model(marie2->lc,presence);
 	/*wait for new status*/
 	wait_for_list(lcs,&pauline->stat.number_of_NotifyReceived,5,2000);
-	CU_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf)); /*because liblinphone compositor is very simple for now (I.E only take first occurence)*/
+	BC_ASSERT_EQUAL(LinphoneStatusBusy,linphone_friend_get_status(lf), int, "%d"); /*because liblinphone compositor is very simple for now (I.E only take first occurence)*/
 
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(marie2);
@@ -477,17 +559,13 @@ test_t presence_tests[] = {
 	{ "Unsubscribe while subscribing", unsubscribe_while_subscribing },
 	{ "Presence information", presence_information },
 	{ "App managed presence failure", subscribe_failure_handle_by_app },
+	{ "Presence SUBSCRIBE forked", subscribe_presence_forked },
+	{ "Presence SUBSCRIBE expired", subscribe_presence_expired },
 #if USE_PRESENCE_SERVER
 	{ "Subscribe with late publish", test_subscribe_notify_publish },
 	{ "Forked subscribe with late publish", test_forked_subscribe_notify_publish },
 #endif
 };
 
-test_suite_t presence_test_suite = {
-	"Presence",
-	NULL,
-	NULL,
-	sizeof(presence_tests) / sizeof(presence_tests[0]),
-	presence_tests
-};
-
+test_suite_t presence_test_suite = {"Presence", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
+									sizeof(presence_tests) / sizeof(presence_tests[0]), presence_tests};
