@@ -16,8 +16,7 @@
 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <stdio.h>
-#include "CUnit/Basic.h"
+
 #include "linphonecore.h"
 #include "private.h"
 #include "liblinphone_tester.h"
@@ -37,7 +36,7 @@ static int test_stun_encode( char*buffer, size_t len, bool_t expect_fail )
 	memset(&username,0,sizeof(username));
 	memset(&password,0,sizeof(password));
 	stunBuildReqSimple( &req, &username, TRUE , TRUE , 11);
-	len = stunEncodeMessage( &req, buffer, len, &password);
+	len = stunEncodeMessage( &req, buffer, (unsigned int)len, &password);
 	if (len<=0){
 		if( expect_fail )
 			ms_message("Fail to encode stun message (EXPECTED).\n");
@@ -45,11 +44,11 @@ static int test_stun_encode( char*buffer, size_t len, bool_t expect_fail )
 			ms_error("Fail to encode stun message.\n");
 		return -1;
 	}
-	return len;
+	return (int)len;
 }
 
 
-static void linphone_stun_test_encode()
+static void linphone_stun_test_encode(void)
 {
 	char smallBuff[12];
 	size_t smallLen = 12;
@@ -57,15 +56,15 @@ static void linphone_stun_test_encode()
 	size_t bigLen = STUN_MAX_MESSAGE_SIZE;
 
 	size_t len = test_stun_encode(smallBuff, smallLen, TRUE);
-	CU_ASSERT(len == -1);
+	BC_ASSERT(len == -1);
 
 	len = test_stun_encode(bigBuff, bigLen, TRUE);
-	CU_ASSERT(len > 0);
+	BC_ASSERT(len > 0);
 	ms_message("STUN message encoded in %i bytes", (int)len);
 }
 
 
-static void linphone_stun_test_grab_ip()
+static void linphone_stun_test_grab_ip(void)
 {
 	LinphoneCoreManager* lc_stun = linphone_core_manager_new2( "stun_rc", FALSE);
 	LinphoneCall dummy_call;
@@ -73,25 +72,31 @@ static void linphone_stun_test_grab_ip()
 	int tmp=0;
 
 	memset(&dummy_call, 0, sizeof(LinphoneCall));
-	dummy_call.media_ports[0].rtp_port = 7078;
-	dummy_call.media_ports[1].rtp_port = 9078;
+	dummy_call.main_audio_stream_index = 0;
+	dummy_call.main_video_stream_index = 1;
+	dummy_call.main_text_stream_index = 2;
+	dummy_call.media_ports[dummy_call.main_audio_stream_index].rtp_port = 7078;
+	dummy_call.media_ports[dummy_call.main_video_stream_index].rtp_port = 9078;
+	dummy_call.media_ports[dummy_call.main_text_stream_index].rtp_port = 11078;
 
 	linphone_core_set_stun_server(lc_stun->lc, stun_address);
-	CU_ASSERT_STRING_EQUAL(stun_address, linphone_core_get_stun_server(lc_stun->lc));
+	BC_ASSERT_STRING_EQUAL(stun_address, linphone_core_get_stun_server(lc_stun->lc));
 
 	wait_for(lc_stun->lc,lc_stun->lc,&tmp,1);
 
 	ping_time = linphone_core_run_stun_tests(lc_stun->lc, &dummy_call);
-	CU_ASSERT(ping_time != -1);
+	BC_ASSERT(ping_time != -1);
 
 	ms_message("Round trip to STUN: %d ms", ping_time);
 
-	CU_ASSERT( dummy_call.ac.addr[0] != '\0');
-	CU_ASSERT( dummy_call.ac.port != 0);
+	BC_ASSERT( dummy_call.ac.addr[0] != '\0');
+	BC_ASSERT( dummy_call.ac.port != 0);
 #ifdef VIDEO_ENABLED
-	CU_ASSERT( dummy_call.vc.addr[0] != '\0');
-	CU_ASSERT( dummy_call.vc.port != 0);
+	BC_ASSERT( dummy_call.vc.addr[0] != '\0');
+	BC_ASSERT( dummy_call.vc.port != 0);
 #endif
+	BC_ASSERT( dummy_call.tc.addr[0] != '\0');
+	BC_ASSERT( dummy_call.tc.port != 0);
 
 	ms_message("STUN test result: local audio port maps to %s:%i",
 			dummy_call.ac.addr,
@@ -101,6 +106,9 @@ static void linphone_stun_test_grab_ip()
 			dummy_call.vc.addr,
 			dummy_call.vc.port);
 #endif
+	ms_message("STUN test result: local text port maps to %s:%i",
+			dummy_call.tc.addr,
+			dummy_call.tc.port);
 
 	linphone_core_manager_destroy(lc_stun);
 }
@@ -111,10 +119,5 @@ test_t stun_tests[] = {
 	{ "STUN encode buffer protection", linphone_stun_test_encode },
 };
 
-test_suite_t stun_test_suite = {
-	"Stun",
-	NULL,
-	NULL,
-	sizeof(stun_tests) / sizeof(stun_tests[0]),
-	stun_tests
-};
+test_suite_t stun_test_suite = {"Stun", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
+								sizeof(stun_tests) / sizeof(stun_tests[0]), stun_tests};
