@@ -17,19 +17,18 @@
 */
 #include "private.h"
 
-#if defined(VIDEO_ENABLED) && defined(HAVE_GTK)
+#if defined(VIDEO_ENABLED)
+
 
-#include <stdio.h>
-#include "CUnit/Basic.h"
 #include "linphonecore.h"
 #include "liblinphone_tester.h"
 #include "lpconfig.h"
 
-
+#if HAVE_GTK
 #include <gtk/gtk.h>
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
-#elif defined(WIN32)
+#elif defined(_WIN32)
 #include <gdk/gdkwin32.h>
 #elif defined(__APPLE__)
 extern void *gdk_quartz_window_get_nswindow(GdkWindow      *window);
@@ -39,13 +38,13 @@ extern void *gdk_quartz_window_get_nsview(GdkWindow      *window);
 #include <gdk/gdkkeysyms.h>
 
 
-static unsigned long get_native_handle(GdkWindow *gdkw) {
+static void *get_native_handle(GdkWindow *gdkw) {
 #ifdef GDK_WINDOWING_X11
-	return (unsigned long)GDK_WINDOW_XID(gdkw);
-#elif defined(WIN32)
-	return (unsigned long)GDK_WINDOW_HWND(gdkw);
+	return (void *)GDK_WINDOW_XID(gdkw);
+#elif defined(_WIN32)
+	return (void *)GDK_WINDOW_HWND(gdkw);
 #elif defined(__APPLE__)
-	return (unsigned long)gdk_quartz_window_get_nsview(gdkw);
+	return (void *)gdk_quartz_window_get_nsview(gdkw);
 #endif
 	g_warning("No way to get the native handle from gdk window");
 	return 0;
@@ -168,24 +167,24 @@ static bool_t video_call_with_params(LinphoneCoreManager* caller_mgr, LinphoneCo
 	bool_t result = TRUE;
 	bool_t did_received_call;
 
-	CU_ASSERT_PTR_NOT_NULL(linphone_core_invite_address_with_params(caller_mgr->lc, callee_mgr->identity, caller_params));
+	BC_ASSERT_PTR_NOT_NULL(linphone_core_invite_address_with_params(caller_mgr->lc, callee_mgr->identity, caller_params));
 	did_received_call = wait_for(callee_mgr->lc, caller_mgr->lc, &callee_mgr->stat.number_of_LinphoneCallIncomingReceived, initial_callee.number_of_LinphoneCallIncomingReceived + 1);
 	if (!did_received_call) return 0;
 
-	CU_ASSERT_EQUAL(caller_mgr->stat.number_of_LinphoneCallOutgoingProgress, initial_caller.number_of_LinphoneCallOutgoingProgress + 1);
+	BC_ASSERT_EQUAL(caller_mgr->stat.number_of_LinphoneCallOutgoingProgress, initial_caller.number_of_LinphoneCallOutgoingProgress + 1, int, "%d");
 
 	while (caller_mgr->stat.number_of_LinphoneCallOutgoingRinging != (initial_caller.number_of_LinphoneCallOutgoingRinging + 1)
 		&& caller_mgr->stat.number_of_LinphoneCallOutgoingEarlyMedia != (initial_caller.number_of_LinphoneCallOutgoingEarlyMedia + 1)
-		&& retry++ < 20) {
+		&& retry++ < 200) {
 		linphone_core_iterate(caller_mgr->lc);
 		linphone_core_iterate(callee_mgr->lc);
-		ms_usleep(100000);
+		ms_usleep(10000);
 	}
 
-	CU_ASSERT_TRUE((caller_mgr->stat.number_of_LinphoneCallOutgoingRinging == initial_caller.number_of_LinphoneCallOutgoingRinging + 1)
+	BC_ASSERT_TRUE((caller_mgr->stat.number_of_LinphoneCallOutgoingRinging == initial_caller.number_of_LinphoneCallOutgoingRinging + 1)
 		|| (caller_mgr->stat.number_of_LinphoneCallOutgoingEarlyMedia == initial_caller.number_of_LinphoneCallOutgoingEarlyMedia + 1));
 
-	CU_ASSERT_PTR_NOT_NULL(linphone_core_get_current_call_remote_address(callee_mgr->lc));
+	BC_ASSERT_PTR_NOT_NULL(linphone_core_get_current_call_remote_address(callee_mgr->lc));
 	if(!linphone_core_get_current_call(caller_mgr->lc) || !linphone_core_get_current_call(callee_mgr->lc) || !linphone_core_get_current_call_remote_address(callee_mgr->lc)) {
 		return 0;
 	}
@@ -193,8 +192,8 @@ static bool_t video_call_with_params(LinphoneCoreManager* caller_mgr, LinphoneCo
 	if (automatically_accept == TRUE) {
 		linphone_core_accept_call_with_params(callee_mgr->lc, linphone_core_get_current_call(callee_mgr->lc), callee_params);
 
-		CU_ASSERT_TRUE(wait_for(callee_mgr->lc, caller_mgr->lc, &callee_mgr->stat.number_of_LinphoneCallConnected, initial_callee.number_of_LinphoneCallConnected + 1));
-		CU_ASSERT_TRUE(wait_for(callee_mgr->lc, caller_mgr->lc, &caller_mgr->stat.number_of_LinphoneCallConnected, initial_callee.number_of_LinphoneCallConnected + 1));
+		BC_ASSERT_TRUE(wait_for(callee_mgr->lc, caller_mgr->lc, &callee_mgr->stat.number_of_LinphoneCallConnected, initial_callee.number_of_LinphoneCallConnected + 1));
+		BC_ASSERT_TRUE(wait_for(callee_mgr->lc, caller_mgr->lc, &caller_mgr->stat.number_of_LinphoneCallConnected, initial_callee.number_of_LinphoneCallConnected + 1));
 		result = wait_for(callee_mgr->lc, caller_mgr->lc, &caller_mgr->stat.number_of_LinphoneCallStreamsRunning, initial_caller.number_of_LinphoneCallStreamsRunning + 1)
 			&& wait_for(callee_mgr->lc, caller_mgr->lc, &callee_mgr->stat.number_of_LinphoneCallStreamsRunning, initial_callee.number_of_LinphoneCallStreamsRunning + 1);
 	}
@@ -248,35 +247,35 @@ static void early_media_video_during_video_call_test(void) {
 	LinphoneCallParams *laure_params;
 
 	marie = linphone_core_manager_new("marie_rc");
-	pauline = linphone_core_manager_new("pauline_rc");
+	pauline = linphone_core_manager_new("pauline_tcp_rc");
 	laure = linphone_core_manager_new("laure_rc");
 	marie_params = configure_for_early_media_video_receiving(marie);
 	pauline_params = configure_for_video(pauline);
 	laure_params = configure_for_early_media_video_sending(laure);
 
 	/* Normal automatically accepted video call from marie to pauline. */
-	CU_ASSERT_TRUE(video_call_with_params(marie, pauline, marie_params, pauline_params, TRUE));
+	BC_ASSERT_TRUE(video_call_with_params(marie, pauline, marie_params, pauline_params, TRUE));
 
 	/* Wait for 2s. */
 	wait_for_three_cores(marie->lc, pauline->lc, NULL, 2000);
 
 	/* Early media video call from laure to marie. */
-	CU_ASSERT_TRUE(video_call_with_params(laure, marie, laure_params, NULL, FALSE));
+	BC_ASSERT_TRUE(video_call_with_params(laure, marie, laure_params, NULL, FALSE));
 
 	/* Wait for 2s. */
 	wait_for_three_cores(marie->lc, pauline->lc, laure->lc, 2000);
 
 	linphone_core_terminate_all_calls(marie->lc);
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallEnd, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallReleased, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallReleased, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, laure->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, laure->lc, &laure->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallReleased, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallReleased, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, laure->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, laure->lc, &laure->stat.number_of_LinphoneCallEnd, 1));
 
-	CU_ASSERT_EQUAL(marie->stat.number_of_video_windows_created, 2);
-	CU_ASSERT_EQUAL(pauline->stat.number_of_video_windows_created, 1);
-	CU_ASSERT_EQUAL(laure->stat.number_of_video_windows_created, 1);
+	BC_ASSERT_EQUAL(marie->stat.number_of_video_windows_created, 2, int, "%d");
+	BC_ASSERT_EQUAL(pauline->stat.number_of_video_windows_created, 1, int, "%d");
+	BC_ASSERT_EQUAL(laure->stat.number_of_video_windows_created, 1, int, "%d");
 
 	linphone_call_params_unref(marie_params);
 	linphone_call_params_unref(pauline_params);
@@ -298,7 +297,7 @@ static void two_incoming_early_media_video_calls_test(void) {
 	const MSList *calls_list;
 
 	marie = linphone_core_manager_new("marie_rc");
-	pauline = linphone_core_manager_new("pauline_rc");
+	pauline = linphone_core_manager_new("pauline_tcp_rc");
 	laure = linphone_core_manager_new("laure_rc");
 	marie_params = configure_for_early_media_video_receiving(marie);
 	pauline_params = configure_for_early_media_video_sending(pauline);
@@ -306,27 +305,27 @@ static void two_incoming_early_media_video_calls_test(void) {
 
 	/* Configure early media audio to play ring during early-media and send remote ring back tone. */
 	linphone_core_set_ring_during_incoming_early_media(marie->lc, TRUE);
-	ringback_path = ms_strdup_printf("%s/sounds/ringback.wav", bc_tester_read_dir_prefix);
+	ringback_path = bc_tester_res("sounds/ringback.wav");
 	linphone_core_set_remote_ringback_tone(marie->lc, ringback_path);
 	ms_free(ringback_path);
 
 	/* Early media video call from pauline to marie. */
-	CU_ASSERT_TRUE(video_call_with_params(pauline, marie, pauline_params, NULL, FALSE));
+	BC_ASSERT_TRUE(video_call_with_params(pauline, marie, pauline_params, NULL, FALSE));
 
 	/* Wait for 2s. */
 	wait_for_three_cores(marie->lc, pauline->lc, NULL, 2000);
 
 	/* Early media video call from laure to marie. */
-	CU_ASSERT_TRUE(video_call_with_params(laure, marie, laure_params, NULL, FALSE));
+	BC_ASSERT_TRUE(video_call_with_params(laure, marie, laure_params, NULL, FALSE));
 
 	/* Wait for 2s. */
 	wait_for_three_cores(marie->lc, pauline->lc, laure->lc, 2000);
 
-	CU_ASSERT_EQUAL(linphone_core_get_calls_nb(marie->lc), 2);
+	BC_ASSERT_EQUAL(linphone_core_get_calls_nb(marie->lc), 2, int, "%d");
 	if (linphone_core_get_calls_nb(marie->lc) == 2) {
 		calls_list = linphone_core_get_calls(marie->lc);
 		call = (LinphoneCall *)ms_list_nth_data(calls_list, 0);
-		CU_ASSERT_PTR_NOT_NULL(call);
+		BC_ASSERT_PTR_NOT_NULL(call);
 		if (call != NULL) {
 			LinphoneCallParams *params = linphone_call_params_copy(linphone_call_get_current_params(call));
 			linphone_call_params_set_audio_direction(params, LinphoneMediaDirectionSendRecv);
@@ -339,16 +338,16 @@ static void two_incoming_early_media_video_calls_test(void) {
 	}
 
 	linphone_core_terminate_all_calls(marie->lc);
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallEnd, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallReleased, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallReleased, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, laure->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, laure->lc, &laure->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallReleased, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallReleased, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, laure->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, laure->lc, &laure->stat.number_of_LinphoneCallEnd, 1));
 
-	CU_ASSERT_EQUAL(marie->stat.number_of_video_windows_created, 2);
-	CU_ASSERT_EQUAL(pauline->stat.number_of_video_windows_created, 1);
-	CU_ASSERT_EQUAL(laure->stat.number_of_video_windows_created, 1);
+	BC_ASSERT_EQUAL(marie->stat.number_of_video_windows_created, 2, int, "%d");
+	BC_ASSERT_EQUAL(pauline->stat.number_of_video_windows_created, 1, int, "%d");
+	BC_ASSERT_EQUAL(laure->stat.number_of_video_windows_created, 1, int, "%d");
 
 	linphone_call_params_unref(marie_params);
 	linphone_call_params_unref(pauline_params);
@@ -365,28 +364,28 @@ static void early_media_video_with_inactive_audio(void) {
 	LinphoneCallParams *pauline_params;
 
 	marie = linphone_core_manager_new("marie_rc");
-	pauline = linphone_core_manager_new("pauline_rc");
+	pauline = linphone_core_manager_new("pauline_tcp_rc");
 	marie_params = configure_for_early_media_video_receiving_with_inactive_audio(marie);
 	pauline_params = configure_for_early_media_video_sending(pauline);
 
 	/* Early media video call from pauline to marie. */
-	CU_ASSERT_TRUE(video_call_with_params(pauline, marie, pauline_params, NULL, FALSE));
+	BC_ASSERT_TRUE(video_call_with_params(pauline, marie, pauline_params, NULL, FALSE));
 
 	/* Wait for 2s. */
 	wait_for_three_cores(marie->lc, pauline->lc, NULL, 2000);
 
 	/* Check that we are in LinphoneCallOutgoingEarlyMedia state and that the ringstream is present meaning we are playing the ringback tone. */
-	CU_ASSERT_EQUAL(linphone_call_get_state(linphone_core_get_current_call(pauline->lc)), LinphoneCallOutgoingEarlyMedia);
-	CU_ASSERT_PTR_NOT_NULL(pauline->lc->ringstream);
+	BC_ASSERT_EQUAL(linphone_call_get_state(linphone_core_get_current_call(pauline->lc)), LinphoneCallOutgoingEarlyMedia, int, "%d");
+	BC_ASSERT_PTR_NOT_NULL(pauline->lc->ringstream);
 
 	linphone_core_terminate_all_calls(marie->lc);
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallEnd, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallReleased, 1));
-	CU_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallReleased, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallEnd, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallReleased, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallReleased, 1));
 
-	CU_ASSERT_EQUAL(marie->stat.number_of_video_windows_created, 1);
-	CU_ASSERT_EQUAL(pauline->stat.number_of_video_windows_created, 1);
+	BC_ASSERT_EQUAL(marie->stat.number_of_video_windows_created, 1, int, "%d");
+	BC_ASSERT_EQUAL(pauline->stat.number_of_video_windows_created, 1, int, "%d");
 
 	linphone_call_params_unref(marie_params);
 	linphone_call_params_unref(pauline_params);
@@ -432,52 +431,68 @@ static void forked_outgoing_early_media_video_call_with_inactive_audio_test(void
 	linphone_core_invite_address_with_params(pauline->lc, marie1->identity, pauline_params);
 	linphone_call_params_destroy(pauline_params);
 
-	CU_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallIncomingEarlyMedia, 1, 3000));
-	CU_ASSERT_TRUE(wait_for_list(lcs, &marie2->stat.number_of_LinphoneCallIncomingEarlyMedia, 1, 3000));
-	CU_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallOutgoingEarlyMedia, 1, 3000));
+	BC_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallIncomingReceived, 1, 3000));
+	BC_ASSERT_TRUE(wait_for_list(lcs, &marie2->stat.number_of_LinphoneCallIncomingReceived, 1, 3000));
 
-	pauline_call = linphone_core_get_current_call(pauline->lc);
 	marie1_call = linphone_core_get_current_call(marie1->lc);
 	marie2_call = linphone_core_get_current_call(marie2->lc);
 
-	CU_ASSERT_PTR_NOT_NULL(pauline_call);
-	CU_ASSERT_PTR_NOT_NULL(marie1_call);
-	CU_ASSERT_PTR_NOT_NULL(marie2_call);
+	if (marie1_call){
+		linphone_call_set_next_video_frame_decoded_callback(marie1_call, linphone_call_iframe_decoded_cb, marie1->lc);
+	}
+	if (marie2_call){
+		linphone_call_set_next_video_frame_decoded_callback(marie2_call, linphone_call_iframe_decoded_cb, marie2->lc);
+	}
+
+	BC_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallIncomingEarlyMedia, 1, 3000));
+	BC_ASSERT_TRUE(wait_for_list(lcs, &marie2->stat.number_of_LinphoneCallIncomingEarlyMedia, 1, 3000));
+	BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallOutgoingEarlyMedia, 1, 3000));
+
+	pauline_call = linphone_core_get_current_call(pauline->lc);
+
+	BC_ASSERT_PTR_NOT_NULL(pauline_call);
+	BC_ASSERT_PTR_NOT_NULL(marie1_call);
+	BC_ASSERT_PTR_NOT_NULL(marie2_call);
 
 	if (pauline_call && marie1_call && marie2_call) {
+		linphone_call_set_next_video_frame_decoded_callback(pauline_call, linphone_call_iframe_decoded_cb, pauline->lc);
+
 		/* wait a bit that streams are established */
-		wait_for_list(lcs, &dummy, 1, 6000);
-		CU_ASSERT_TRUE(linphone_call_get_audio_stats(pauline_call)->download_bandwidth == 0);
-		CU_ASSERT_TRUE(linphone_call_get_audio_stats(marie1_call)->download_bandwidth == 0);
-		CU_ASSERT_TRUE(linphone_call_get_audio_stats(marie2_call)->download_bandwidth == 0);
-		CU_ASSERT_TRUE(linphone_call_get_video_stats(pauline_call)->download_bandwidth == 0);
-		CU_ASSERT_TRUE(linphone_call_get_video_stats(marie1_call)->download_bandwidth > 0);
-		CU_ASSERT_TRUE(linphone_call_get_video_stats(marie2_call)->download_bandwidth > 0);
+		wait_for_list(lcs, &dummy, 1, 3000);
+		BC_ASSERT_EQUAL(linphone_call_get_audio_stats(pauline_call)->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_EQUAL(linphone_call_get_audio_stats(marie1_call)->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_EQUAL(linphone_call_get_audio_stats(marie2_call)->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_LOWER(linphone_call_get_video_stats(pauline_call)->download_bandwidth, 11, float, "%f"); /* because of stun packets*/
+		BC_ASSERT_GREATER(linphone_call_get_video_stats(marie1_call)->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_GREATER(linphone_call_get_video_stats(marie2_call)->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_GREATER(marie1->stat.number_of_IframeDecoded, 1, int, "%i");
+		BC_ASSERT_GREATER(marie2->stat.number_of_IframeDecoded, 1, int, "%i");
 
 		linphone_call_params_set_audio_direction(marie1_params, LinphoneMediaDirectionSendRecv);
 		linphone_core_accept_call_with_params(marie1->lc, linphone_core_get_current_call(marie1->lc), marie1_params);
-		CU_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallStreamsRunning, 1, 3000));
-		CU_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallStreamsRunning, 1, 3000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallStreamsRunning, 1, 3000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallStreamsRunning, 1, 3000));
 
 		/* marie2 should get her call terminated */
-		CU_ASSERT_TRUE(wait_for_list(lcs, &marie2->stat.number_of_LinphoneCallEnd, 1, 1000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &marie2->stat.number_of_LinphoneCallEnd, 1, 1000));
 
 		/*wait a bit that streams are established*/
 		wait_for_list(lcs, &dummy, 1, 3000);
-		CU_ASSERT_TRUE(linphone_call_get_audio_stats(pauline_call)->download_bandwidth > 71);
-		CU_ASSERT_TRUE(linphone_call_get_audio_stats(marie1_call)->download_bandwidth > 71);
-		CU_ASSERT_TRUE(linphone_call_get_video_stats(pauline_call)->download_bandwidth > 0);
-		CU_ASSERT_TRUE(linphone_call_get_video_stats(marie1_call)->download_bandwidth > 0);
+		BC_ASSERT_GREATER(linphone_call_get_audio_stats(pauline_call)->download_bandwidth, 71, float, "%f");
+		BC_ASSERT_GREATER(linphone_call_get_audio_stats(marie1_call)->download_bandwidth, 71, float, "%f");
+		BC_ASSERT_GREATER(linphone_call_get_video_stats(pauline_call)->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_GREATER(linphone_call_get_video_stats(marie1_call)->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_GREATER(pauline->stat.number_of_IframeDecoded, 1, int, "%i");
 
 		/* send an INFO in reverse side to check that dialogs are properly established */
 		info = linphone_core_create_info_message(marie1->lc);
 		linphone_call_send_info_message(marie1_call, info);
-		CU_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_inforeceived, 1, 3000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_inforeceived, 1, 3000));
 	}
 
 	linphone_core_terminate_all_calls(pauline->lc);
-	CU_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallEnd, 1, 3000));
-	CU_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallEnd, 1, 3000));
+	BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallEnd, 1, 3000));
+	BC_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallEnd, 1, 3000));
 
 	linphone_call_params_destroy(marie1_params);
 	linphone_call_params_destroy(marie2_params);
@@ -486,20 +501,58 @@ static void forked_outgoing_early_media_video_call_with_inactive_audio_test(void
 	linphone_core_manager_destroy(marie2);
 	linphone_core_manager_destroy(pauline);
 }
+#endif /*HAVE_GTK*/
 
+static void enable_disable_camera_after_camera_switches(void) {
+	LinphoneCoreManager* marie = linphone_core_manager_new("marie_rc");
+	LinphoneCoreManager* pauline = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	const char *currentCamId = (char*)linphone_core_get_video_device(marie->lc);
+	const char **cameras=linphone_core_get_video_devices(marie->lc);
+	const char *newCamId=NULL;
+	int i;
+
+
+	video_call_base_2(marie,pauline,FALSE,LinphoneMediaEncryptionNone,TRUE,TRUE);
+
+	for (i=0;cameras[i]!=NULL;++i){
+		if (strcmp(cameras[i],currentCamId)!=0){
+			newCamId=cameras[i];
+			break;
+		}
+	}
+	if (newCamId){
+		LinphoneCall *call = linphone_core_get_current_call(marie->lc);
+		ms_message("Switching from [%s] to [%s]", currentCamId, newCamId);
+		linphone_core_set_video_device(marie->lc, newCamId);
+		if(call != NULL) {
+			linphone_core_update_call(marie->lc, call, NULL);
+		}
+		BC_ASSERT_STRING_EQUAL(newCamId,ms_web_cam_get_string_id(linphone_call_get_video_device(call)));
+		linphone_call_enable_camera(call,FALSE);
+		linphone_core_iterate(marie->lc);
+		linphone_call_enable_camera(call,TRUE);
+		BC_ASSERT_STRING_EQUAL(newCamId,ms_web_cam_get_string_id(linphone_call_get_video_device(call)));
+	}
+
+
+	linphone_core_terminate_all_calls(pauline->lc);
+	BC_ASSERT_TRUE(wait_for(pauline->lc,marie->lc,&pauline->stat.number_of_LinphoneCallEnd,1));
+	BC_ASSERT_TRUE(wait_for(pauline->lc,marie->lc,&marie->stat.number_of_LinphoneCallEnd,1));
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline);
+}
 test_t video_tests[] = {
+#if HAVE_GTK
 	{ "Early-media video during video call", early_media_video_during_video_call_test },
 	{ "Two incoming early-media video calls", two_incoming_early_media_video_calls_test },
 	{ "Early-media video with inactive audio", early_media_video_with_inactive_audio },
-	{ "Forked outgoing early-media video call with inactive audio", forked_outgoing_early_media_video_call_with_inactive_audio_test }
+	{ "Forked outgoing early-media video call with inactive audio", forked_outgoing_early_media_video_call_with_inactive_audio_test },
+#endif /*HAVE_GTK*/
+	{ "Enable/disable camera after camera switches", enable_disable_camera_after_camera_switches}
+
 };
 
-test_suite_t video_test_suite = {
-	"Video",
-	NULL,
-	NULL,
-	sizeof(video_tests) / sizeof(video_tests[0]),
-	video_tests
-};
+test_suite_t video_test_suite = {"Video", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
+								 sizeof(video_tests) / sizeof(video_tests[0]), video_tests};
 
 #endif /* VIDEO_ENABLED */
